SystemOrganization addCategory: #'Path-Tests-Callgraph'!
SystemOrganization addCategory: #'Path-Tests-Harvester'!
SystemOrganization addCategory: #'Path-Tests-Harvester-Sandbox'!
SystemOrganization addCategory: #'Path-Tests-MethodWrappers'!
SystemOrganization addCategory: #'Path-Tests-PathFinder'!
SystemOrganization addCategory: #'Path-Tests-PathView'!
SystemOrganization addCategory: #'Path-Tests-PathView-Sandbox'!
SystemOrganization addCategory: #'Path-Tests-Projects'!
SystemOrganization addCategory: #'Path-Tests-Record-Replay'!
SystemOrganization addCategory: #'Path-Tests-Sandbox'!
SystemOrganization addCategory: #'Path-Tests-Tracer'!

TestCase subclass: #SBrowserExtensionTest
	instanceVariableNames: 'oldCurrentProject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathFinder'!

!SBrowserExtensionTest methodsFor: 'testing' stamp: 'mp 3/23/2011 14:59:20.509'!
openBrowserOnClass: aClass selector: aSelector
	
	| brow classToUse |
	classToUse := SystemBrowser default.
	brow := classToUse new.
	brow setClass: aClass selector: aSelector.
	^ classToUse 
		openBrowserView: (brow openEditString: nil)
		label: brow labelString! !

!SBrowserExtensionTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:51:50.264'!
setUp

	oldCurrentProject := SProject current.
	SProject setCurrent: #sandbox.! !

!SBrowserExtensionTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:51:53.575'!
tearDown

	SProject setCurrentProject: oldCurrentProject.! !

!SBrowserExtensionTest methodsFor: 'testing' stamp: 'mp 7/8/2010 17:59:15.977'!
testBrowseBehavior

	| browser testExpander |
	browser := self openBrowserOnClass: SSBClassA selector: #doNothing.
	testExpander := browser openPathFinderOn: (MethodReference class: SSBTest selector: #testDoNothing).
	self assert: testExpander notNil.
	self assert: testExpander testChooserModel isTestAvailable.
	self assert: testExpander testChooserModel test actualClass == SSBTest.
	self assert: testExpander testChooserModel test methodSymbol == #testDoNothing.
	self assert: testExpander callgraphModel selectedItem clientClass = SSBClassA.
	self assert: testExpander callgraphModel selectedItem clientSelector = #doNothing.
	self assert: testExpander callgraphModel searchQuery = 'SSBClassA>>doNothing'.
	browser dependents first abandon.
	testExpander windowMorph abandon.! !

!SBrowserExtensionTest methodsFor: 'testing' stamp: 'mp 2/1/2012 10:47:27.232'!
testEntryPointsOfClassADoNothing

	| browser tests |
	browser := self openBrowserOnClass: SSBClassA selector: #doNothing.
	tests := browser coveredTestsSortedFor: (MethodReference class: SSBClassA selector: #doNothing).
	self assert: tests size = 17.
	browser dependents first abandon.! !

TestCase subclass: #SCallNodeTest
	instanceVariableNames: 'rootNode calledNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

!SCallNodeTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 10:59'!
calledNode
	"Answer the value of calledNode"

	^ calledNode! !

!SCallNodeTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 10:59'!
calledNode: anObject
	"Set the value of calledNode"

	calledNode := anObject! !

!SCallNodeTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 10:59'!
rootNode
	"Answer the value of rootNode"

	^ rootNode! !

!SCallNodeTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 10:59'!
rootNode: anObject
	"Set the value of rootNode"

	rootNode := anObject! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:25.551'!
setUp

	| callgraph |
	callgraph := SCallgraphModel new.
	self rootNode: (SCallNode id: 0 class: SSBTest methodName: #testDoNothing fromSender: nil model: callgraph receiverType: SSBTest callsBefore: 0).
	self calledNode: (SCallNode id: 1 class: SSBClassA methodName: #doNothing fromSender: self rootNode model: callgraph receiverType: SSBClassA callsBefore: 0).! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 3/9/2010 11:07'!
testAddCall

	self rootNode addCall: self calledNode.
	self assert: self rootNode nextCalls notEmpty.
	self assert: self rootNode nextCalls first == self calledNode.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 3/9/2010 11:06'!
testFromRootSender

	self assert: self calledNode fromSender == self rootNode.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 3/9/2010 11:11'!
testGetCallStackFromMultipleCalledNodes

	self rootNode addCall: self calledNode.
	self rootNode addCall: self calledNode.
	self rootNode addCall: self calledNode.
	self assert: self rootNode nextCalls size = 3.
	self assert: (self rootNode nextCalls second getCallStack = (OrderedCollection with: self rootNode with: self calledNode)).! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 3/9/2010 11:12'!
testGetCallStackFromRoot

	self assert: self rootNode getCallStack = (OrderedCollection with: self rootNode).! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:47.397'!
testInitializationRootNode

	self assert: self rootNode id = 0.
	self assert: self rootNode fromSender = nil.
	self assert: self rootNode nextCalls isEmpty.
	self assert: self rootNode selectedView = #classes.
	self assert: self rootNode selectedProfile = #normal.
	self assert: self rootNode views notEmpty.
	self assert: (self rootNode views at: #classes) = SSBTest.
	self assert: self rootNode profiles isEmpty.
	self assert: self rootNode clientSelector == #testDoNothing.
	self assert: self rootNode clientClass == SSBTest.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:40:49.541'!
testPrintProfile

	self rootNode selectedProfile: #%.
	self rootNode profiles at: #% put: '100%'.
	self assert: self rootNode asString =  '[100%]SSBTest>>testDoNothing'.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:40:56.338'!
testPrintString

	self assert: self rootNode asString =  'SSBTest>>testDoNothing'.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 4/24/2010 17:55'!
testPrintView

	self rootNode selectedView: #objects.
	self rootNode views at: #objects put: 'a Test'.
	self assert: self rootNode asString =  'a Test>>testDoNothing'.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 4/24/2010 17:55'!
testPrintViewAndProfile

	self rootNode selectedView: #objects.
	self rootNode views at: #objects put: 'a Test'.
	self rootNode selectedProfile: #%.
	self rootNode profiles at: #% put: '100%'.
	self assert: self rootNode asString =  '[100%]a Test>>testDoNothing'.! !

!SCallNodeTest methodsFor: 'testing' stamp: 'mp 3/9/2010 11:18'!
testSetAsCurrentSelection

	self assert: self rootNode callgraphModel selectedItem isNil.
	self rootNode setAsCurrentSelection.
	self assert: self rootNode callgraphModel selectedItem = self rootNode.! !

TestCase subclass: #SPathFinderTest
	instanceVariableNames: 'oldCurrentProject browserModel browserMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathFinder'!

!SPathFinderTest methodsFor: 'accessing' stamp: 'mp 1/31/2010 18:59'!
browserModel
	"Answer the value of browserModel"

	^ browserModel! !

!SPathFinderTest methodsFor: 'accessing' stamp: 'mp 1/31/2010 18:59'!
browserModel: anObject
	"Set the value of browserModel"

	browserModel := anObject! !

!SPathFinderTest methodsFor: 'accessing' stamp: 'mp 1/31/2010 18:59'!
browserMorph
	"Answer the value of browserMorph"

	^ browserMorph! !

!SPathFinderTest methodsFor: 'accessing' stamp: 'mp 1/31/2010 18:59'!
browserMorph: anObject
	"Set the value of browserMorph"

	browserMorph := anObject! !

!SPathFinderTest methodsFor: 'accessing' stamp: 'mp 3/14/2010 19:14'!
naviBar

	^ self browserMorph submorphs fourth! !

!SPathFinderTest methodsFor: 'testing' stamp: 'mp 7/8/2010 17:20:53.514'!
setUp

	oldCurrentProject := SProject current.
	SProject setCurrent: #sandbox.
	self browserModel: SPathFinder new.
	self browserMorph: (ToolBuilder build: self browserModel).
		
	! !

!SPathFinderTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:53:35.567'!
tearDown

	SProject setCurrentProject: oldCurrentProject.! !

SPathFinderTest subclass: #SPathFinderTestChooserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathFinder'!

!SPathFinderTestChooserTest methodsFor: 'accessing' stamp: 'mp 3/12/2010 17:18'!
getTestModel
	
	^ self browserModel testChooserModel! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:54.441'!
testAnalyzeTest

	self assert: self getTestModel result isNil.
	self assert: self browserModel callgraphModel rootNode isNil.
	self getTestModel setCurrentTest: (MethodReference class: SSBTest selector: #testDoNothing).
	self getTestModel analyzeTest.
	self assert: self getTestModel result hasPassed.
	self assert: self browserModel callgraphModel rootNode notNil.! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:56.285'!
testDeselectTest

	self getTestModel setCurrentTest: (MethodReference class: SSBTest selector: #testDoNothing).
	self getTestModel setCurrentTest: nil.
	self assert: self browserModel callgraphModel rootNode isNil.! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:52.501'!
testRerunTestButtonEnabled

	self deny: self getTestModel isTestAvailable.
	self getTestModel setCurrentTest: (MethodReference class: SSBTest selector: #testDoNothing).
	self assert: self getTestModel isTestAvailable.
	self getTestModel setCurrentTest: nil.
	self deny: self getTestModel isTestAvailable.! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:50.293'!
testSelectAnotherTest

	self getTestModel setCurrentTest: (MethodReference class: SSBTest selector: #testDoNothing).
	self assert: self browserModel callgraphModel rootNode clientSelector == #testDoNothing.
	self getTestModel setCurrentTest: (MethodReference class: SSBTest selector: #testBinary).
	self assert: self browserModel callgraphModel rootNode clientSelector == #testBinary.! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 3/14/2010 14:05'!
testSelectProfiling

	self assert: self getTestModel tracer profilingMode == #normal.
	self getTestModel normalRunWithCounts.
	self assert: self getTestModel tracer profilingMode == #counts.
	self getTestModel profileRunWithMs.
	self assert: self getTestModel tracer profilingMode == #ms.
	self getTestModel profileRunWithPercentage.
	self assert: self getTestModel tracer profilingMode == #%.! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:57.973'!
testSelectTest

	self getTestModel setCurrentTest: (MethodReference class: SSBTest selector: #testDoNothing).
	self assert: self browserModel callgraphModel rootNode notNil.
	self getTestModel setCurrentTest: nil.
	self assert: self browserModel callgraphModel rootNode isNil.! !

!SPathFinderTestChooserTest methodsFor: 'testing' stamp: 'mp 3/14/2010 14:04'!
testSelectView

	self assert: self getTestModel tracer viewMode == #classes.
	self getTestModel selectObjectView.
	self assert: self getTestModel tracer viewMode == #objects.
	self getTestModel selectIdentityView.
	self assert: self getTestModel tracer viewMode == #identities.! !

SPathFinderTest subclass: #SPathFinderToolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathFinder'!

!SPathFinderToolTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:53:40.696'!
testModelCreation

	self assert: self browserModel notNil.
	self assert: self browserModel callgraphTracer notNil.
	self assert: self browserModel callgraphModel notNil.
	self assert: self browserModel testChooserModel notNil.
	
	self assert: self browserModel callgraphTracer selectedCategories = SProject current categories.
	self assert: self browserModel callgraphTracer callgraphModel == self browserModel callgraphModel.
	self assert: self browserModel callgraphModel tracer == self browserModel callgraphTracer.
	
	self assert: self browserModel testChooserModel tracer == self browserModel callgraphTracer.! !

!SPathFinderToolTest methodsFor: 'testing' stamp: 'mp 3/24/2011 11:39:13.43'!
testMorphCreation

	self assert: self browserMorph notNil.
	
	self assert: (self browserMorph submorphs at: 1) class == SDropDownMorph.
	self assert: (self browserMorph submorphs at: 1) actionModel == self browserModel testChooserModel.	
	self assert: (self browserMorph submorphs at: 1) labelMorph contents = 'Without Profiling'.
	
	self assert: (self browserMorph submorphs at: 2) class == SDropDownMorph.
	self assert: (self browserMorph submorphs at: 2) actionModel == self browserModel testChooserModel.
	self assert: (self browserMorph submorphs at: 2) labelMorph contents = 'Class View'.

	self assert: (self browserMorph submorphs at: 3) class == STestResultFeedbackMorph.
	self assert: (self browserMorph submorphs at: 3) actionModel == self browserModel.

	self assert: (self browserMorph submorphs at: 5) class == PluggablePanelMorph.
	self assert: (self browserMorph submorphs at: 5) submorphs size = 2.
	self assert: (self browserMorph submorphs at: 5) submorphs first label = '<<'.
	self assert: (self browserMorph submorphs at: 5) submorphs second label = '>>'.

	self assert: (self browserMorph submorphs at: 6) class == PluggableTextMorphPlus.
	self assert: (self browserMorph submorphs at: 6) knownName = 'searchInputField'.
	self assert: (self browserMorph submorphs at: 6) model == self browserModel callgraphModel.

	self assert: (self browserMorph submorphs at: 7) class == SPluggableTreeMorph.
	self assert: (self browserMorph submorphs at: 7) knownName = 'callGraphTree'.
	self assert: (self browserMorph submorphs at: 7) model == self browserModel callgraphModel.
	
"feedback morph ist nue und muss noch getestet werden"! !

TestCase subclass: #SProjectTest
	instanceVariableNames: 'oldRegistry oldCurrentProject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Projects'!

SProjectTest subclass: #SProjectAccessTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Projects'!

!SProjectAccessTest methodsFor: 'running' stamp: 'mp 3/22/2013 11:25:11.779'!
setUp
	super setUp.
	SProject addProject: SProject observerExampleProject.
	SProject setCurrent: #observerExample.! !

!SProjectAccessTest methodsFor: 'class tests' stamp: 'mp 3/22/2013 11:27:31.328'!
test01AllClasses

	self assert: self project allClasses =  {ClockTimer. Driver. Observer. SAnalogClock. SDigitalClock. SVisibleClock. Subject. TestDriver}.! !

!SProjectAccessTest methodsFor: 'category tests' stamp: 'mp 3/22/2013 11:32:05.387'!
test01CategoriesWithTest

	self assert: self project categoriesWithTests = 
					#(#'Path-Dev-Example-Observer-Tests').! !

!SProjectAccessTest methodsFor: 'method category tests' stamp: 'mp 3/22/2013 11:37:37.075'!
test01MethodCategoriesOfClass

	self assert: (self project methodCategoriesOfClass: Observer) =   #(#accessing #observation #'instance creation (class)').	! !

!SProjectAccessTest methodsFor: 'print tests' stamp: 'mp 3/22/2013 15:07:11.493'!
test01PrintMetricsOverview

	self assert: self project metricsOverviewString = 
		'2 Categories, 8 Classes, and 30 Methods (136 LoC)'.! !

!SProjectAccessTest methodsFor: 'project tests' stamp: 'mp 3/22/2013 11:46:50.483'!
test01ProjectInitialized

	self assert: self project notNil.
	self assert: self project projectName == #observerExample.! !

!SProjectAccessTest methodsFor: 'method tests' stamp: 'mp 3/22/2013 13:41:42.834'!
test01aMethodsPerClass
	
	self assert: (self project methodsPerClass: ClockTimer) = 
		#(#hour #hour: #initialize #minute #minute: #second #second: #tick)! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 11:41:09.429'!
test01aNomOfClass

	self assert: (self project nomOfClass: Driver) = 2.
	self assert: (self project nomOfClass: Subject) = 4.
	self assert: (self project nomOfClass: ClockTimer) = 8.! !

!SProjectAccessTest methodsFor: 'method tests' stamp: 'mp 3/22/2013 13:42:01.558'!
test01bMethodsPerClass
	
	self assert: (self project methodsPerClass: ClockTimer withQuick: false) = 
		#(#hour: #initialize #minute: #second: #tick)! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 11:42:35.276'!
test01bNomOfClassWithoutQuick

	self assert: (self project nomOfClass: Driver withQuick: false) = 2.
	self assert: (self project nomOfClass: Subject withQuick: false) = 4.
	self assert: (self project nomOfClass: ClockTimer withQuick: false) = 5.! !

!SProjectAccessTest methodsFor: 'category tests' stamp: 'mp 3/22/2013 11:32:36.916'!
test02CategoriesOrderedByTests

	self assert: self project categoriesOrderedByTestsFirst =  #(#'Path-Dev-Example-Observer-Tests' #'Path-Dev-Example-Observer').! !

!SProjectAccessTest methodsFor: 'method category tests' stamp: 'mp 3/22/2013 11:48:23.82'!
test02MethodsOfMethodCategoryAndClass

	self assert: (self project methodsPerClass: Subject andMethodCategory: #observation) =   #(#attach: #detach: #notify).
	self assert: (self project methodsPerClass: SVisibleClock andMethodCategory: #'instance creation (class)') =   #(#newOn:inFrame: #on:).! !

!SProjectAccessTest methodsFor: 'print tests' stamp: 'mp 3/22/2013 15:13:24.118'!
test02PrintString

	self assert: self project asString = 'observerExample <current>'.
	SProject addProject: SProject sandboxProject.
	self assert: (SProject projectAt: #sandbox) asString = 'sandbox'.! !

!SProjectAccessTest methodsFor: 'project tests' stamp: 'mp 6/11/2010 16:26'!
test02ResetClassesCache

	self assert: (self project instVarNamed: 'classesCache') isNil.
	self project allClasses.
	self assert: (self project instVarNamed: 'classesCache') notNil.
	self project resetClassesCache.
	self assert: (self project instVarNamed: 'classesCache') isNil.	! !

!SProjectAccessTest methodsFor: 'class tests' stamp: 'mp 3/22/2013 11:28:33.464'!
test02TestClasses

	self assert: self project testClasses =  {TestDriver}! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 11:51:54.186'!
test02aNomOfClassAndMethodCategory

	self assert: (self project 
					nomOfClass: ClockTimer 
					methodCategory: 'accessing') = 6.
	self assert: (self project 
					nomOfClass: ClockTimer 
					methodCategory: 'initialize-release') = 1.
! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 15:03:22.008'!
test02bNomOfClassAndMethodCategoryWithoutQuick

	self assert: (self project 
					nomOfClass: ClockTimer 
					methodCategory: 'accessing' withQuick: false) = 3.
	self assert: (self project 
					nomOfClass: ClockTimer 
					methodCategory: 'initialize-release' withQuick: false) = 1.
! !

!SProjectAccessTest methodsFor: 'class tests' stamp: 'mp 3/22/2013 11:54:22.496'!
test03ClassesOfCategory

	self assert: (self project classesOfCategory: #'Path-Dev-Example-Observer') =  
	{ClockTimer. Driver. Observer. SAnalogClock. SDigitalClock. SVisibleClock. Subject}.! !

!SProjectAccessTest methodsFor: 'print tests' stamp: 'mp 3/22/2013 15:14:42.276'!
test03FullPrintString

	self assert: self project fullPrintString = 'a SProject <observerExample>'.! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 11:56:04.692'!
test03NomOfCategory

	self assert: (self project nomOfCategory: #'Path-Dev-Example-Observer') = 27.
	self assert: (self project categories inject: 0 into: [:sum :cat | 
				sum := sum + (self project nomOfCategory: cat)]) = self project nom.! !

!SProjectAccessTest methodsFor: 'project tests' stamp: 'mp 3/22/2013 13:37:14.821'!
test03RemoveClassFromCache

	self project allClasses.
	self assert: (self project instVarNamed: 'classesCache') includes: Observer.
	self project removeFromClassesCache: Observer.
	self deny: (self project instVarNamed: 'classesCache') includes: Observer.
	! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 15:02:39.828'!
test04NomOfProject

	self assert: self project nom = 30.
	self assert: self project nom = (self project allClasses 
									inject: 0 
									into: [:sum :each | 
										sum + each methodDict size + each class methodDict size])! !

!SProjectAccessTest methodsFor: 'class tests' stamp: 'mp 3/22/2013 11:55:16.952'!
test04TestClassesOfCategory

	self assert: (self project 
		testClassesOfCategory: #'Path-Dev-Example-Observer-Tests') =  {TestDriver}.
	self assert: (self project 
		testClassesOfCategory: #'Path-Dev-Example-Observer') isEmpty.! !

!SProjectAccessTest methodsFor: 'class tests' stamp: 'mp 3/22/2013 13:43:06.099'!
test05AllClassesWithoutTests

	self assert: self project allClassesWithoutTests = 
	{ClockTimer. Driver. Observer. SAnalogClock. SDigitalClock. SVisibleClock. Subject}.! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 15:02:50.467'!
test05NocOfProject

	self assert: self project noc = 8.! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 15:02:59.152'!
test06NopOfProject

	self assert: self project nop = 2.! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 15:03:07.566'!
test07LocOfProject

	self assert: self project loc =  136.! !

!SProjectAccessTest methodsFor: 'metric tests' stamp: 'mp 3/22/2013 15:04:33.827'!
test08NomWithoutLowComplexity

	self assert: self project nomWithoutLowComplexity = 6.! !

SProjectTest subclass: #SProjectBucketTest
	instanceVariableNames: 'pathMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Projects'!

!SProjectBucketTest methodsFor: 'accessing' stamp: 'mp 3/22/2013 14:20:52.95'!
pathMap
	"Answer the value of pathMap"

	^ pathMap! !

!SProjectBucketTest methodsFor: 'accessing' stamp: 'mp 3/22/2013 14:20:53.127'!
pathMap: anObject
	"Set the value of pathMap"

	pathMap := anObject! !

!SProjectBucketTest methodsFor: 'running' stamp: 'mp 3/22/2013 14:52:39.808'!
runInductiveAnalysisWith: aHarvesterClass
	self pathMap model pathMap selectedOptions	setKindOfAnalysis: aHarvesterClass.
	self pathMap model runAll.! !

!SProjectBucketTest methodsFor: 'running' stamp: 'mp 3/22/2013 14:31:21.061'!
setUp
	super setUp.
	SProject addProject: SProject observerExampleProject.
	SProject setCurrent: #observerExample.
	self pathMap: SPathMapTestRunner open.
	(self pathMap model getFlap: #inductive) switchExpandState.
	self pathMap model pathMap selectedAnalysis: #inductive.! !

!SProjectBucketTest methodsFor: 'running' stamp: 'mp 3/22/2013 14:20:53.151'!
tearDown

	self pathMap abandon.
	super tearDown.! !

!SProjectBucketTest methodsFor: 'testing' stamp: 'mp 3/22/2013 14:52:21.664'!
test01RunHarvesting
	self assert: self project harvestedBuckets isEmpty.
	self runInductiveAnalysisWith: THTypeHarvester.
	self assert: self project harvestedBuckets notEmpty.! !

!SProjectBucketTest methodsFor: 'testing' stamp: 'mp 3/22/2013 14:52:54.518'!
test02TypeBucket
	self assert: self project typesBucket isNil.
	self runInductiveAnalysisWith: THTypeHarvester.
	self assert: self project valueRangeBucket isNil.
	self assert: self project typesBucket notNil.
	self assert: (self project typesBucket class inheritsFrom: THTypeBucket)! !

!SProjectBucketTest methodsFor: 'testing' stamp: 'mp 3/22/2013 14:53:01.327'!
test03ValueRangeBucket
	self assert: self project typesBucket isNil.
	self runInductiveAnalysisWith: SValueRangeHarvester.
	self assert: self project typesBucket isNil.
	self assert: self project valueRangeBucket notNil.
	self assert: (self project valueRangeBucket class == SValueRangeBucket)! !

!SProjectBucketTest methodsFor: 'testing' stamp: 'mp 3/22/2013 14:56:15.267'!
test04BothBuckets
	self runInductiveAnalysisWith: THTypeHarvester.
	self runInductiveAnalysisWith: SValueRangeHarvester.
	self assert: self project typesBucket notNil.
	self assert: self project valueRangeBucket notNil.! !

SProjectTest subclass: #SProjectCoverageTest
	instanceVariableNames: 'pathMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Projects'!

!SProjectCoverageTest methodsFor: 'accessing' stamp: 'mp 3/22/2013 13:48:11.062'!
pathMap
	"Answer the value of pathMap"

	^ pathMap! !

!SProjectCoverageTest methodsFor: 'accessing' stamp: 'mp 3/22/2013 13:48:11.074'!
pathMap: anObject
	"Set the value of pathMap"

	pathMap := anObject! !

!SProjectCoverageTest methodsFor: 'running' stamp: 'mp 3/22/2013 14:30:12.94'!
setUp
	super setUp.
	SProject addProject: SProject observerExampleProject.
	SProject setCurrent: #observerExample.
	self pathMap: SPathMapTestRunner open.
	(self pathMap model getFlap: #testQuality)	switchExpandState.
	self pathMap model pathMap selectedAnalysis: #testQuality.
	self pathMap model runAll.! !

!SProjectCoverageTest methodsFor: 'running' stamp: 'mp 3/22/2013 14:06:05.746'!
tearDown

	self pathMap abandon.
	super tearDown.! !

!SProjectCoverageTest methodsFor: 'testing' stamp: 'mp 3/22/2013 14:06:38.51'!
test01RunCoverage

	self assert: self project coverage notEmpty.! !

!SProjectCoverageTest methodsFor: 'testing' stamp: 'mp 3/22/2013 14:19:19.905'!
test02CoveredTestsForMethod
	"covered by one test"
	self assert: (self project coveredTestsFor: 
		(MethodReference class: SVisibleClock selector: #initInFrame:)) asArray = 		{(STestCaseEntryPoint class: TestDriver selector: #testRun).}.
	"covered by multiple tests"
	self assert: (self project coveredTestsFor: 
		(MethodReference class: TestDriver selector: #tearDown)) asArray = 
		{(STestCaseEntryPoint class: TestDriver selector: #testRun). 
		(STestCaseEntryPoint class: TestDriver selector: #testNoop).}.
	"Not covered"
	self assert: (self project coveredTestsFor: 
		(MethodReference class: Driver selector: #run)) asArray = {}.
	"Not in project"
	self assert: (self project coveredTestsFor: 
		(MethodReference class: SProjectTest selector: #tearDown)) asArray = {}.
! !

SProjectTest subclass: #SProjectRegistryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Projects'!

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:53:13.09'!
test01EmptyProjectRegistry

	self assert: SProject projectRegistry isEmpty.
	self assert: SProject allProjects isEmpty.
	self assert: SProject allProjectNames isEmpty.! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:52:44.789'!
test02ProjectRegistryInitalizeWithStandardProjects

	SProject addPredefinedProjects.
	self deny: SProject projectRegistry isEmpty.
	self assert: (SProject projectAt: #NeverUseThisSymbolAsAProjectyxnbcvejg) isNil.
	self assert: (SProject projectAt: #sunit) notNil.! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:52:32.351'!
test03aProjectRegistryAtMonticelloProject
	| project |
	SProject addPredefinedProjects.
	project := SProject projectAt: #monticello.
	self assert: project notNil.
	self assert: project projectName == #monticello.
	self assert: project categories = #(#'Monticello-Base' #'Monticello-Chunk Format' #'Monticello-Loading' #'Monticello-Merging' #'Monticello-Modeling' #'Monticello-Patching' #'Monticello-Repositories' #'Monticello-Storing' #'Monticello-UI' #'Monticello-Versioning' #'Tests-Monticello' #'Tests-Monticello-Mocks').! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:52:57.294'!
test03bProjectRegistryAtSUnitProject
	| project |
	SProject addPredefinedProjects.
	project := SProject projectAt: #sunit.
	self assert: project notNil.
	self assert: project projectName == #sunit.
	self assert: project categories = #(#'SUnit-Extensions' #'SUnit-Kernel' #'SUnit-Tests')! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:53:01.969'!
test03cProjectRegistryAtSystemBrowserProject
	| project |
	SProject addPredefinedProjects.
	project := SProject projectAt: #systemBrowser.
	self assert: project notNil.
	self assert: project projectName == #systemBrowser.
	self assert: project categories = #(#'ToolBuilder-Kernel' #'ToolBuilder-Morphic' #'Tools-Base' #'Tools-Browser' #'ToolsTests-Browser')! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:52:23.366'!
test04AddNewProject
	| project |
	project := SProject name: #myProject categories: #(#'DontExists').
	self assert: (SProject projectAt: #myProject) isNil.
	SProject addProject: project.
	self assert: (SProject projectAt: #myProject) notNil.	! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 3/22/2013 11:58:36.951'!
test05RemoveProject

	SProject addPredefinedProjects.
	SProject removeProjectAt: #observerExample.
	self assert: (SProject projectAt: #observerExample) isNil.! !

!SProjectRegistryTest methodsFor: 'tests' stamp: 'mp 7/7/2010 16:53:18.927'!
test06ResetProjectRegistry

	SProject addPredefinedProjects.
	SProject reset.
	self assert: SProject projectRegistry isEmpty.! !

!SProjectTest methodsFor: 'as yet unclassified' stamp: 'mp 3/22/2013 13:59:09.814'!
project

	^ SProject current! !

!SProjectTest methodsFor: 'running' stamp: 'mp 7/7/2010 16:52:00.248'!
setUp

	oldRegistry := SProject projectRegistry.
	oldCurrentProject := SProject current.
	SProject reset.! !

!SProjectTest methodsFor: 'running' stamp: 'mp 7/7/2010 16:52:04.062'!
tearDown
	SProject projectRegistry: oldRegistry.
	SProject setCurrentProject: oldCurrentProject.! !

TestCase subclass: #SRecordAndReplayTest
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Record-Replay'!

!SRecordAndReplayTest methodsFor: 'accessing' stamp: 'mp 5/18/2010 11:48'!
cache
	"Answer the value of cache"

	^ cache! !

!SRecordAndReplayTest methodsFor: 'accessing' stamp: 'mp 5/18/2010 11:48'!
cache: anObject
	"Set the value of cache"

	cache := anObject! !

!SRecordAndReplayTest methodsFor: 'running' stamp: 'mp 7/7/2010 16:58:36.095'!
setUp

	self cache: SReplayCache new.! !

!SRecordAndReplayTest methodsFor: 'running' stamp: 'mp 7/7/2010 16:57:54.603'!
tearDown

	SRecordReplayWrapper nuke.
	Smalltalk garbageCollect.! !

!SRecordAndReplayTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:57:51.617'!
testRecordDateAndTimeNow
	| time |
	(SRecordReplayWrapper on: #now inClass: DateAndTime class with: self cache) install.
	time := DateAndTime now.
	(Delay forSeconds: 1) wait.
	self assert: time = DateAndTime now.! !

!SRecordAndReplayTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:57:58.805'!
testRecordDateAndTimeNowUninstallWrapper
	| time wrapper |
	wrapper := (SRecordReplayWrapper on: #now inClass: DateAndTime class with: self cache) install.
	time := DateAndTime now.
	(Delay forSeconds: 1) wait.
	self assert: time = DateAndTime now.
	wrapper uninstall.
	self deny: time = DateAndTime now.! !

!SRecordAndReplayTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:58:05.997'!
testRecordMultipleRandoms

	| result1 result2 |
	(SRecordReplayWrapper on: #atRandom inClass: Integer with: self cache) install.
	result1 := 10000 atRandom.
	result2 := 9999 atRandom.
	self assert: 10000 atRandom = result1.
	self assert: 9999 atRandom = result2.! !

!SRecordAndReplayTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:58:02.317'!
testRecordRandom

	| result |
	(SRecordReplayWrapper on: #atRandom inClass: Integer with: self cache) install.
	result := 10000 atRandom.
	self assert: 10000 atRandom = result.! !

TestCase subclass: #SSBFaultLocalizationExampleTest
	instanceVariableNames: 'example'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBFaultLocalizationExampleTest methodsFor: 'accessing' stamp: 'mp 9/21/2011 14:23:30.654'!
example
	"Answer the value of example"

	^ example! !

!SSBFaultLocalizationExampleTest methodsFor: 'accessing' stamp: 'mp 9/21/2011 14:23:30.67'!
example: anObject
	"Set the value of example"

	example := anObject! !

!SSBFaultLocalizationExampleTest methodsFor: 'running' stamp: 'mp 9/21/2011 14:28:22.142'!
setUp
	self example: SSBFaultLocalizationExample new.! !

!SSBFaultLocalizationExampleTest methodsFor: 'testing' stamp: 'mp 9/21/2011 14:36:07.623'!
test123
	self example midX: 1 Y: 2 Z: 3.
	self assert: (self example mid) = 2.! !

!SSBFaultLocalizationExampleTest methodsFor: 'testing' stamp: 'mp 2/1/2012 11:26:10.884'!
test213
	"Expected Failure for Testing Fault Localization - do not add to #expectedFailures"
	self example midX: 2 Y: 1 Z: 3.
	self assert: (self example mid) = 2.! !

!SSBFaultLocalizationExampleTest methodsFor: 'testing' stamp: 'mp 9/21/2011 14:36:29.724'!
test321
	self example midX: 3 Y: 2 Z: 1.
	self assert: (self example mid) = 2.! !

!SSBFaultLocalizationExampleTest methodsFor: 'testing' stamp: 'mp 9/21/2011 14:36:39.001'!
test335
	self example midX: 3 Y: 3 Z: 5.
	self assert: (self example mid) = 3.! !

!SSBFaultLocalizationExampleTest methodsFor: 'testing' stamp: ''!
test534
	self example midX: 5 Y: 3 Z: 4.
	self assert: (self example mid) = 4.! !

!SSBFaultLocalizationExampleTest methodsFor: 'testing' stamp: 'mp 9/21/2011 14:37:00.606'!
test555
	self example midX: 5 Y: 5 Z: 5.
	self assert: (self example mid) = 5.! !

TestCase subclass: #SSBFaultLocalizationTest
	instanceVariableNames: 'a'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBFaultLocalizationTest methodsFor: 'accessing' stamp: 'mp 11/5/2010 10:04:18.32'!
a
	"Answer the value of a"

	^ a! !

!SSBFaultLocalizationTest methodsFor: 'accessing' stamp: 'mp 11/5/2010 10:04:18.325'!
a: anObject
	"Set the value of a"

	a := anObject! !

!SSBFaultLocalizationTest methodsFor: 'as yet unclassified' stamp: 'mp 11/5/2010 14:10:33.885'!
setUp

	self a: SSBClassA new.! !

!SSBFaultLocalizationTest methodsFor: 'as yet unclassified' stamp: 'mp 2/1/2012 11:26:26.044'!
testFailingMethodFault
	"Expected Failure for Testing Fault Localization - do not add to #expectedFailures"
	self deny: (self a failingMethod: false).! !

!SSBFaultLocalizationTest methodsFor: 'as yet unclassified' stamp: 'mp 11/5/2010 14:12:16.308'!
testFailingMethodPass

	self assert: (self a failingMethod: true).
	self a unaryMethod.! !

!SSBFaultLocalizationTest methodsFor: 'as yet unclassified' stamp: 'mp 11/5/2010 16:50:03.059'!
xtestFailingMethodError
	self a doNothing.
	self deny: (self a failingMethod: 1)! !

TestCase subclass: #SSBTest
	instanceVariableNames: 'a'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBTest methodsFor: 'accessing' stamp: 'mp 3/8/2010 14:35'!
a
	"Answer the value of a"

	^ a! !

!SSBTest methodsFor: 'accessing' stamp: 'mp 3/8/2010 14:35'!
a: anObject
	"Set the value of a"

	a := anObject! !

!SSBTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:28.17'!
setUp

	self a: SSBClassA new.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 3/8/2010 14:35'!
tearDown

	self a doNothing.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 3/8/2010 14:38'!
testBinary

	self assert: self a + 5 = 5.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 1/24/2012 15:21:40.139'!
testComplexTrace
	self assert: (self a complexMethod: 5) = 10.
	! !

!SSBTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:12.736'!
testConstructor

	self assert: (SSBClassB on: 1) myValue = 1.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 1/24/2012 15:21:35.298'!
testDelay
	<ignoreForHarvesting>
	"Required for Profiling"
	self assert: self a delayedMethod.
	! !

!SSBTest methodsFor: 'testing' stamp: 'mp 3/8/2010 14:38'!
testDoNothing

	self assert: self a doNothing = self a.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 3/8/2010 14:34'!
testInitialize

	self assert: self a myValue = 0.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 11/8/2010 17:01:53.424'!
testLongParameterMethod
	self a param: 1 method: 2.
	self a manyParameters: 1 b: 2 d: 3 f: 4 long: 5! !

!SSBTest methodsFor: 'testing' stamp: 'mp 5/8/2012 11:55:52.523'!
testMethodChain

	self assert: (self a methodChain: (self a methodChain: (self a methodChain: 1) + 1) + 1) = 3.
	! !

!SSBTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:14.617'!
testModelView

	| b |
	b := SSBClassB new.
	self a myValue: 1.
	self assert: b myValue isNil.
	self a addDependent: b.
	self a myValue: 2.
	self assert: b myValue = 2.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 11/8/2010 15:37:20.516'!
testMultipleByteCodePaths

	self assert: (self a param: 5 method: 2) = 4.
	self assert: (self a param: 3 method: 2) = #(3 2).! !

!SSBTest methodsFor: 'testing' stamp: ''!
testOwnCollectionType
	| dict |
	dict := SSBDictionary in: 123.
	dict symbolFor: #abc.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 3/8/2010 14:40'!
testParamMethod

	self assert: (self a param: 1 method: 2) = #(1 2).
	self assert: self a myValue = #(1 2).! !

!SSBTest methodsFor: 'testing' stamp: 'mp 2/1/2012 11:26:33.429'!
testParamMethod2
	"Expected Failure for Testing Fault Localization - do not add to #expectedFailures"
	self assert: (self a param: 5 method: 2) = 5.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:32:14.64'!
testPartial

	self assert: self a doNothing = self a.
	self assert: SPartialTraceObject new doSomething.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 5/9/2012 10:52:35.499'!
testPolymorphie

	self assert: (SSBClassC new complexMethod: 5) = 10! !

!SSBTest methodsFor: 'testing' stamp: 'mp 1/24/2012 16:08:04.563'!
testPrintString
	<ignore>
	self a myValue: 5.
	self assert: self a asString = '5'.
	! !

!SSBTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:30.165'!
testRecursion

	self assert: (SSBClassA new recursionMethod: 5) = 15.! !

!SSBTest methodsFor: 'testing' stamp: 'mp 3/8/2010 14:37'!
testUnary

	self assert: self a unaryMethod.! !

TestCase subclass: #SSBTypeTest
	instanceVariableNames: 'a'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBTypeTest methodsFor: 'as yet unclassified' stamp: 'mp 1/21/2013 15:14:46.012'!
testCollection
	| d |
	d := SSBClassD new.
	d myValue2: {1. 2. 3.}.! !

!SSBTypeTest methodsFor: 'as yet unclassified' stamp: ''!
testDictionary
	| d |
	d := SSBClassD new.
	d myValue3: (Dictionary new at: 1 put: #abc; yourself).! !

!SSBTypeTest methodsFor: 'as yet unclassified' stamp: 'mp 1/21/2013 15:14:25.203'!
testObject
	| d |
	d := SSBClassD new.
	d myValue: 2.! !

TestCase subclass: #SSBValueRangeTest
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBValueRangeTest methodsFor: 'running' stamp: 'mp 2/6/2012 16:00:35.311'!
setUp
	object := SSBClassD new.! !

!SSBValueRangeTest methodsFor: 'test-boolean' stamp: ''!
testBoolean

	object myValue: true.
	object myValue2: false.
	object myValue3: nil.! !

!SSBValueRangeTest methodsFor: 'test-characters' stamp: 'mp 2/8/2012 14:52:26.164'!
testCharacters1

	object myValue: $1.	
	object myValue: $a.
	object myValue: $C.	
	object myValue: Character space.
	object myValue: $?.! !

!SSBValueRangeTest methodsFor: 'test-characters' stamp: 'mp 2/8/2012 14:52:56.135'!
testCharacters2

	object myValue2: $Z.
	object myValue2: $a.	
	object myValue2: nil.! !

!SSBValueRangeTest methodsFor: 'test-characters' stamp: 'mp 2/8/2012 14:54:29.859'!
testCharacters3

	object myValue3: $.! !

!SSBValueRangeTest methodsFor: 'test-collections' stamp: 'mp 2/10/2012 09:59:48.086'!
testCollection1

	object myValue: {1. 2. 3}.
	object myValue: {4. 5. 6. 7}.	
	object myValue: {}.	! !

!SSBValueRangeTest methodsFor: 'test-collections' stamp: 'mp 2/10/2012 10:00:49.178'!
testCollection2

	object myValue2: {1. 2. 3}.
	object myValue2: {4. 5. 6. 4} asSet.	
	object myValue2: (Dictionary newFrom: {1->2. 2->3. 3->4}).	! !

!SSBValueRangeTest methodsFor: 'test-datetime' stamp: 'mp 2/10/2012 11:35:02.087'!
testDateAndTime1

	object myValue: DateAndTime now.! !

!SSBValueRangeTest methodsFor: 'test-datetime' stamp: 'mp 2/10/2012 11:35:38.812'!
testDateAndTime2

	object myValue2: DateAndTime tomorrow.
	object myValue2: nil.
	object myValue2: DateAndTime today.
	object myValue2: DateAndTime yesterday.! !

!SSBValueRangeTest methodsFor: 'test-duration' stamp: 'mp 2/10/2012 12:08:53.759'!
testDuration1

	object myValue: (Duration seconds: 10).
	object myValue: (Duration seconds: -10).
	object myValue: (Duration seconds: 0).! !

!SSBValueRangeTest methodsFor: 'test-duration' stamp: 'mp 2/10/2012 12:09:25.541'!
testDuration2

	object myValue2: (Duration nanoSeconds: 10).
	object myValue2: (Duration nanoSeconds: -10).
	object myValue2: (Duration nanoSeconds: 0).! !

!SSBValueRangeTest methodsFor: 'test-duration' stamp: 'mp 2/10/2012 12:10:11.441'!
testDuration3

	object myValue3: (Duration days: 1).
	object myValue3: nil.
	object myValue3: (Duration nanoSeconds: 10000).! !

!SSBValueRangeTest methodsFor: 'as yet unclassified' stamp: 'mp 3/30/2012 13:22:59.869'!
testEvil
	object myValue: 0.
	SSBClassA new + 2.3.! !

!SSBValueRangeTest methodsFor: 'test-numbers' stamp: 'mp 2/6/2012 16:04:38.977'!
testNumbers1

	object myValue: 1.	
	object myValue: 3.
	object myValue: 7.	
	object myValue: 10.! !

!SSBValueRangeTest methodsFor: 'test-numbers' stamp: 'mp 2/6/2012 16:04:59.826'!
testNumbers2

	object myValue2: -1.	
	object myValue2: 3.
	object myValue2: 0.	
	object myValue2: -7.	
	object myValue2: 10.! !

!SSBValueRangeTest methodsFor: 'test-numbers' stamp: 'mp 2/6/2012 16:05:35.282'!
testNumbers3

	object myValue3: -1.43.	
	object myValue3: 3.123.
	object myValue3: 0.	
	object myValue3: (-1/3).	
	object myValue3: 10.! !

!SSBValueRangeTest methodsFor: 'test-numbers' stamp: 'mp 2/6/2012 16:06:35.948'!
testNumbers4

	object myValue4: 1.	
	object myValue4: nil.
	object myValue4: 1.			
! !

!SSBValueRangeTest methodsFor: 'test-streams' stamp: 'mp 2/10/2012 14:09:37.364'!
testStreams1

	object myValue: (WriteStream on: String new)! !

!SSBValueRangeTest methodsFor: 'test-streams' stamp: 'mp 2/10/2012 14:11:09.22'!
testStreams2

	object myValue2: (WriteStream on: OrderedCollection new).
	object myValue2: nil.
	object myValue2: (ReadStream on: 'abcdef').! !

!SSBValueRangeTest methodsFor: 'test-strings' stamp: 'mp 2/8/2012 17:32:50.956'!
testString1

	object myValue: 'This is a sentence.'.
! !

!SSBValueRangeTest methodsFor: 'test-strings' stamp: 'mp 2/8/2012 17:35:19.227'!
testString2

	object myValue2: 'This is a sentence with a mitsake.'.	
! !

!SSBValueRangeTest methodsFor: 'test-strings' stamp: 'mp 2/8/2012 17:36:30.209'!
testString3

	object myValue3: '123456'.
	object myValue3: nil.
	object myValue3: 'Content-Lenght?'.
	
! !

!SSBValueRangeTest methodsFor: 'test-strings' stamp: ''!
testString4

	object myValue4: '1 halt.'.
	object myValue4: '/abcdef/ghi.txt'.
	object myValue4: 'This is a sentence.'.
	object myValue4: '12.31.2012'.
	object myValue4: '/Users'.
	
! !

TestCase subclass: #STracerTest
	instanceVariableNames: 'oldCurrentProject tracer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

STracerTest subclass: #SCallgraphTest
	instanceVariableNames: 'callgraph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

SCallgraphTest subclass: #SCallgraphFunctionsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:16.678'!
testBrowseSelectedNode

	| browser |
	self assert: self cg browseMethod == self cg.
	self cg newSelectedItem: (self getNode: 12).
	browser := self cg browseMethod.
	self assert: browser class == SystemBrowser default.
	self assert: browser selectedClass == SSBClassB.
	self assert: browser selectedMessageName == #compute.
	self flag: #mpe. "Find a better way to close the system window"
	browser dependents first delete.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:06'!
testExpandRootNode

	self assert: self cg selectedItem isNil.
	self cg expandRootNode.
	self assert: self cg selectedItem == self rootNode.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:07'!
testExpandSelectedNode

	self cg expandSelectedNode.
	self assert: self cg selectedItem == self rootNode.
	self cg newSelectedItem: (self getNode: 12).
	self cg expandSelectedNode.
	self assert: self cg selectedItem ==(self getNode: 12).! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:33'!
testFindNodeFromRoot

	self assert: (self cg findNodeInSubGraph: 0 startNode: self rootNode) isNil.
	self assert: (self cg findNodeInSubGraph: 12 startNode: self rootNode) id = 12.
	self assert: (self cg findNodeInSubGraph: 22 startNode: self rootNode) id = 22.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:34'!
testFindNodeWithinSubtree

	| node |
	node := self getNode: 12.
	self assert: (self cg findNodeInSubGraph: 0 startNode: node) isNil.
	self assert: (self cg findNodeInSubGraph: 12 startNode: node) isNil.
	self assert: (self cg findNodeInSubGraph: 22 startNode: node) isNil.
	
	self assert: (self cg findNodeInSubGraph: 13 startNode: node) id = 13.
	self assert: (self cg findNodeInSubGraph: 19 startNode: node) id = 19.	! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:35'!
testFindNotExistingNode

	self assert: (self cg findNodeInSubGraph: -1 startNode: self rootNode) isNil.
	self assert: (self cg findNodeInSubGraph: 23 startNode: self rootNode) isNil.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:04'!
testGetSelectedPath

	| path |
	self cg newSelectedItem: (self getNode: 12).
	path := self cg getSelectedPath.
	self assert: path = (OrderedCollection 
								with: self rootNode 
								with: (self getNode: 5) 
								with: (self getNode: 7) 
								with: (self getNode: 12))! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:51'!
testNewSelectedItem

	self assert: self cg selectedItem isNil.
	self cg newSelectedItem: (self getNode: 12).
	self assert: self cg selectedItem notNil.
	self assert: self cg selectedItem == (self getNode: 12).! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:49'!
testNumberOfCallsBefore

	| node |
	self assert: (self cg numberOfCallsBefore: self rootNode) = 0.
	node := self getNode: 12.
	self assert: (self cg numberOfCallsBefore: node) = 0.
	node := self getNode: 13.
	self assert: (self cg numberOfCallsBefore: node) = 0.
	node := self getNode: 15.
	self assert: (self cg numberOfCallsBefore: node) = 1.	
	node := self getNode: 16.
	self assert: (self cg numberOfCallsBefore: node) = 2.	! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:39'!
testTraversCallGraphFromRoot

	| collection |
	collection := OrderedCollection new.
	self cg traverseCallGraph: self rootNode with: [:each | collection add: each id].
	self assert: (1 to: 22) asOrderedCollection = collection.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:41'!
testTraversCallGraphFromRootWithAbort

	| collection |
	collection := OrderedCollection new.
	self cg traverseCallGraph: self rootNode with: [:each | collection add: each id] abort: [:each | each id > 10].
	self assert: #(1 2 3 4 5 6 7 8 9 10 11 12 20) asOrderedCollection = collection.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:42'!
testTraversCallGraphFromSubTree

	| collection node |
	node := self getNode: 12.
	collection := OrderedCollection new.
	self cg traverseCallGraph: node with: [:each | collection add: each id].
	self assert: (13 to: 19) asOrderedCollection = collection.! !

!SCallgraphFunctionsTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:43'!
testTraversCallGraphFromSubTreeWithAbort

	| collection node |
	node := self getNode: 12.
	collection := OrderedCollection new.
	self cg traverseCallGraph: node with: [:each | collection add: each id] abort: [:each | each id > 15].
	self assert: (13 to: 18) asOrderedCollection = collection.! !

SCallgraphTest subclass: #SCallgraphHistoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:10'!
testAddToHistory

	self assert: self cg history isEmpty.
	self assert: self cg currentHistoryNode = 0.
	self cg newSelectedItem: self rootNode.
	self assert: self cg history notEmpty.	
	self assert: self cg history size = 1.
	self assert: self cg history first == self rootNode.
	self assert: self cg currentHistoryNode = 1.! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:11'!
testAddToHistorySameItemTwice

	self cg newSelectedItem: self rootNode.
	self cg newSelectedItem: self rootNode.
	self assert: self cg history notEmpty.	
	self assert: self cg history size = 1.
	self assert: self cg history first == self rootNode.
	self assert: self cg currentHistoryNode = 1.! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:12'!
testAddToHistoryTwoItems

	self cg newSelectedItem: self rootNode.
	self cg newSelectedItem: (self getNode: 12).
	self assert: self cg history notEmpty.	
	self assert: self cg history size = 2.
	self assert: self cg history first == self rootNode.
	self assert: self cg history second == (self getNode: 12).
	self assert: self cg currentHistoryNode = 2.! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:14'!
testHistoryBack

	self cg newSelectedItem: self rootNode.
	self cg newSelectedItem: (self getNode: 12).
	self cg newSelectedItem: (self getNode: 1).
	self cg historyBack.
	self assert: self cg selectedItem == (self getNode: 12).
	self cg historyBack.
	self assert: self cg selectedItem == self rootNode.! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:15'!
testHistoryBackAndForth

	self cg newSelectedItem: self rootNode.
	self cg newSelectedItem: (self getNode: 12).
	self cg newSelectedItem: (self getNode: 1).
	self cg historyBack.
	self assert: self cg selectedItem == (self getNode: 12).
	self cg historyPrevious.
	self assert: self cg selectedItem == (self getNode: 1).! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:17'!
testHistoryBackAndNewSelectedItem

	self cg newSelectedItem: self rootNode.
	self cg newSelectedItem: (self getNode: 12).
	self cg newSelectedItem: (self getNode: 1).
	self cg historyBack.
	self cg historyBack.
	self cg newSelectedItem: (self getNode: 10).
	self assert: self cg history size = 2.
	self assert: self cg history first == self rootNode.
	self assert: self cg history second == (self getNode: 10).
! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:21'!
testHistoryBackButtonEnabled

	self deny: self cg historyBackButtonEnabled.
	self cg newSelectedItem: self rootNode.
	self deny: self cg historyBackButtonEnabled.
	self cg newSelectedItem: (self getNode: 12).
	self assert: self cg historyBackButtonEnabled.
	self cg historyBack.
	self deny: self cg historyBackButtonEnabled.
	self cg historyPrevious.
	self assert: self cg historyBackButtonEnabled.
	! !

!SCallgraphHistoryTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:19'!
testHistoryPreviousButtonEnabled

	self deny: self cg historyPreviousButtonEnabled.
	self cg newSelectedItem: self rootNode.
	self cg newSelectedItem: (self getNode: 12).
	self deny: self cg historyPreviousButtonEnabled.
	self cg historyBack.
	self assert: self cg historyPreviousButtonEnabled.
	self cg historyPrevious.
	self deny: self cg historyPreviousButtonEnabled.! !

SCallgraphTest subclass: #SCallgraphModelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:28'!
testCallgraphCallstackOfNode12

	| node |
	node := self getNode: 12.
	self assert: node getCallStack notEmpty.
	self assert: node getCallStack size = 4.
	self assert: node getCallStack = (OrderedCollection 
								with: self rootNode 
								with: (self getNode: 5) 
								with: (self getNode: 7) 
								with: node)! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:22.72'!
testCallgraphCheckNode12

	| node |
	node := self getNode: 12.
	self assert: node clientClass == SSBClassB.
	self assert: node clientSelector == #compute.	! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:29'!
testCallgraphChildrenOfNode12

	| node |
	node := self getNode: 12.
	self assert: node nextCalls notEmpty.
	self assert: node nextCalls size = 6.
	self assert: node nextCalls = (OrderedCollection 
								with: (self getNode: 13) 
								with: (self getNode: 14) 
								with: (self getNode: 15) 
								with: (self getNode: 16)
								with: (self getNode: 17)
								with: (self getNode: 18))! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:31.892'!
testCallgraphFromSenderOfNode12

	| node |
	node := self getNode: 12.
	self assert: node fromSender clientClass == SSBClassA.
	self assert: node fromSender clientSelector == #complexMethod:.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:09:03.916'!
testCallgraphInitialization

	| testCallgraph |
	testCallgraph := SCallgraphModel on: self tracer.
	self assert: testCallgraph rootNode isNil.
	self assert: testCallgraph currentNode isNil.
	self assert: testCallgraph history isEmpty.
	self assert: testCallgraph currentHistoryNode = 0.
	self assert: testCallgraph selectedItem isNil.
	self assert: testCallgraph highlightedItems isEmpty.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:01.344'!
testCallgraphRootNode

	self assert: self cg rootNode clientClass == SSBTest.
	self assert: self cg rootNode clientSelector == #testComplexTrace.
	self assert: self cg rootNode nextCalls notEmpty.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:16'!
testGetChildren

	self assert: (self cg getChildren: self rootNode) size = 3.
	self assert: (self cg getChildren: (self getNode: 12)) size = 6.
	self assert: (self cg getChildren: (self getNode: 22)) isEmpty.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:10'!
testHasChildren

	self assert: (self cg hasChildren: self rootNode).
	self assert: (self cg hasChildren: (self getNode: 12)).
	self deny: (self cg hasChildren: (self getNode: 22)).! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:13'!
testNumberOfNodesWithinCallgraph

	| number |
	number := 0.
	self cg traverseCallGraph: self cg rootNode with: [:aNode | number := number + 1].
	self assert: number = 22.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:19'!
testReset

	self assert: self cg rootNode notNil.
	self cg reset.
	self assert: self cg rootNode isNil.
	self assert: self cg currentNode isNil.
	self assert: self cg history isEmpty.
	self assert: self cg currentHistoryNode = 0.
	self assert: self cg selectedItem isNil.
	self assert: self cg highlightedItems isEmpty.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:03.696'!
testRootCallNodes

	self assert: self cg rootCallNodes notEmpty.
	self assert: self cg rootCallNodes size = 1.
	self assert: self cg rootCallNodes first clientClass == SSBTest.
	self assert: self cg rootCallNodes first clientSelector == #testComplexTrace.! !

!SCallgraphModelTest methodsFor: 'testing' stamp: 'mp 3/9/2010 13:14'!
testStartingCallNodes

	self assert: self cg startingCallNodes notEmpty.
	self assert: self cg startingCallNodes size = 3.
	self assert: self cg startingCallNodes first clientSelector == #setUp.
	self assert: self cg startingCallNodes second clientSelector == #testComplexTrace.
	self assert: self cg startingCallNodes third clientSelector == #tearDown.! !

SCallgraphTest subclass: #SCallgraphSearchingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:33'!
testGetSearchQuery

	self assert: self cg searchQuery isNil.
	self cg searchNode: 'LSBClassB>>myValue'.
	self assert: self cg searchQuery = 'LSBClassB>>myValue'.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:40:32.144'!
testMultipleItemSearching

	| result |
	result := self cg searchNode: 'SSBClassB>>myValue'.
	self assert: result size = 4.
	self assert: result first id = 11.
	self assert: result second id = 13.
	self assert: result third id = 15.
	self assert: result fourth id = 16.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 3/9/2010 14:29'!
testNotExistingItemSearching

	| result |
	result := self cg searchNode: 'XXX'.
	self assert: result isEmpty.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:41:01.921'!
testSearchingAndHighlighting

	self cg searchNode: 'SSBClassB>>myValue'.
	self assert: self cg highlightedItems size = 4.
	self assert: self cg highlightedItems first id = 11.
	self assert: self cg highlightedItems second id = 13.
	self assert: self cg highlightedItems third id = 15.
	self assert: self cg highlightedItems fourth id = 16.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:40:16.535'!
testSearchingAndSelectFirstResult

	self assert: self cg selectedItem isNil.
	self cg searchNode: 'SSBClassB>>myValue'.
	self assert: self cg selectedItem id = 11.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:40:08.435'!
testSearchingAndSelectNextResult

	self assert: self cg selectedItem isNil.
	self cg searchNode: 'SSBClassB>>myValue'.
	self cg searchNode: 'SSBClassB>>myValue'.
	self assert: self cg selectedItem id = 13.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:41:08.358'!
testSearchingAndSelectNextResultAfterCurrentSelection

	self cg newSelectedItem: (self getNode: 14).
	self cg searchNode: 'SSBClassB>>myValue'.
	self assert: self cg selectedItem id = 15.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:41:13.605'!
testSearchingAndSelectNextResultAfterEndOfTree

	self cg newSelectedItem: (self getNode: 22).
	self cg searchNode: 'SSBClassB>>myValue'.
	self assert: self cg selectedItem id = 11.! !

!SCallgraphSearchingTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:41:18.821'!
testSingleItemSearching

	| result |
	result := self cg searchNode: 'SSBClassB>>compute'.
	self assert: result size = 1.
	self assert: result first = (self getNode: 12).! !

!SCallgraphTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 11:49'!
callgraph
	"Answer the value of callgraph"

	^ callgraph! !

!SCallgraphTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 11:49'!
callgraph: anObject
	"Set the value of callgraph"

	callgraph := anObject! !

!SCallgraphTest methodsFor: 'accessing' stamp: 'mp 3/9/2010 13:06'!
cg
	"Answer the value of callgraph"

	^ self callgraph! !

!SCallgraphTest methodsFor: 'as yet unclassified' stamp: 'mp 3/9/2010 13:11'!
getNode: anId

	^ self callgraph findNodeInSubGraph: anId startNode: self callgraph rootNode! !

!SCallgraphTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:33:59.621'!
setUp

	super setUp.
	self tracer traceTest: SSBTest method: #testComplexTrace.
	self callgraph: self tracer callgraphModel.! !

SCallgraphTest subclass: #SObjectExplorerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Callgraph'!

!SObjectExplorerTest methodsFor: 'testing' stamp: 'mp 4/24/2010 18:43'!
testClearObjectCache

	| callNode |
	callNode := self getNode: 12.
	self assert: callNode objects isEmpty.
	callNode explore: #return.
	self assert: callNode objects notEmpty.
	callNode clearObjectCache.
	self assert: callNode objects isEmpty.! !

!SObjectExplorerTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:25.56'!
testExploreFullCallNode

	| callNode |
	callNode := self getNode: 12.
	self assert: callNode objects isEmpty.
	callNode 
		explore: #types;
		explore: #arguments;
		explore: #selfBefore;
		explore: #selfAfter;
		explore: #return.
	self assert: callNode objects notNil.
	self assert: (callNode objects at: #types) = 
		(OrderedCollection with: #self->SSBClassB with: #result->SmallInteger)."types" 
	self assert: (callNode objects at: #arguments) first = #(). "args" 
	self assert: (callNode objects at: #selfBefore) first myValue myValue = 5. "selfBefore"
	self assert: (callNode objects at: #selfAfter) first myValue myValue = 5. "selfAfter"
	self assert: (callNode objects at: #return) first = 10. "return"
! !

!SObjectExplorerTest methodsFor: 'testing' stamp: 'mp 4/24/2010 19:09'!
testExploreReturnObject

	| callNode |
	callNode := self getNode: 12.
	self assert: callNode objects isEmpty.
	self assert: ((callNode explore: #return) first = 10).
	self assert: callNode objects notEmpty.
	self assert: ((callNode objects at: #return) first = 10).
	callNode callgraphModel: nil. "get cached value without callgraph access - no reload is possible"
	self assert: ((callNode explore: #return) first = 10).! !

!SObjectExplorerTest methodsFor: 'testing' stamp: 'mp 4/24/2010 18:51'!
testInitialization
	
	| callNode |
	callNode := self getNode: 12.
	self assert: callNode tracer == self tracer.
	self assert: callNode objects isEmpty.		! !

STracerTest subclass: #STracerCoreTraceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:42.43'!
testBinaryMethodTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testBinary.
	nodes := self callgraphGetNodesFor: SSBClassA method: #+.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testBinary.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:31.131'!
testClassMethodTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testConstructor.
	nodes := self callgraphGetNodesFor: SSBClassB class method: #on:.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testConstructor.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:40.144'!
testGetNode

	self tracer traceTest: SSBTest method: #testDoNothing.
	self assert: (self tracer getNode: 0) == self rootNode.
	self assert: (self tracer getNode: -1) isNil.
	self assert: (self tracer getNode: 1) == self rootNode nextCalls first.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:35.579'!
testMethodChainTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testMethodChain.
	nodes := self callgraphGetNodesFor: SSBClassA method: #methodChain:.
	self assert: nodes size = 3.
	nodes do: [:each | 
		self assert: each fromSender clientClass == SSBTest.
		self assert: each fromSender clientSelector == #testMethodChain].! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:47.602'!
testModelViewTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testModelView.
	nodes := self callgraphGetNodesFor: SSBClassA method: #myValue:.
	self assert: nodes size = 3.
	self assert: nodes first nextCalls isEmpty.
	self assert: nodes second nextCalls isEmpty.
	self assert: nodes third nextCalls notEmpty.
	self assert: nodes third nextCalls first == (self callgraphGetNodesFor: SSBClassB method: #update:) first.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:54'!
testNextId

	self assert: self tracer currentId = -1.
	self assert: self tracer nextId = 0.
	self assert: self tracer currentId = 0.
	self assert: self tracer nextId = 1.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:33.775'!
testParameterMethodTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testParamMethod.
	nodes := self callgraphGetNodesFor: SSBClassA method: #param:method:.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testParamMethod.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:55.359'!
testPartialTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testPartial.
	nodes := self callgraphGetNodesFor: Behavior method: #new.
	self assert: nodes isEmpty.
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	self assert: nodes notEmpty.
	nodes := self callgraphGetNodesFor: SPartialTraceObject method: #doSomething.
	self assert: nodes isEmpty.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:45.578'!
testRecursionTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testRecursion.
	nodes := self callgraphGetNodesFor: SSBClassA method: #recursionMethod:.
	self assert: nodes size = 6.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testRecursion.
	self assert: nodes second fromSender clientClass == SSBClassA.
	self assert: nodes second fromSender clientSelector == #recursionMethod:.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:00.158'!
testRunSingleWrapper

	| wrappers nodes |
	wrappers := OrderedCollection new.
	self tracer createNewCallgraphOn: SSBTest with: #foobar.
	[self tracer createWrapperOn: SSBClassA and: #doNothing addTo: wrappers.
	SSBClassA new doNothing] 
		ensure: [wrappers first uninstall].
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #foobar.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:26.777'!
testSetUpTestTearDownTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testDoNothing.
	nodes := self callgraphGetNodesFor: SSBTest method: #setUp.
	self assert: nodes size = 1.
	self assert: nodes first fromSender == self rootNode.
	nodes := self callgraphGetNodesFor: SSBTest method: #testDoNothing.
	self assert: nodes size = 1.
	self assert: nodes first fromSender == self rootNode.
	nodes := self callgraphGetNodesFor: SSBTest method: #tearDown.
	self assert: nodes size = 1.
	self assert: nodes first fromSender == self rootNode.! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:40.375'!
testTraceOverClassBorders

	| nodes |
	self tracer traceTest: SSBTest method: #testComplexTrace.
	nodes := self callgraphGetNodesFor: SSBClassA method: #complexMethod:.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testComplexTrace.
	nodes := self callgraphGetNodesFor: SSBClassB method: #compute.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBClassA.
	self assert: nodes first fromSender clientSelector == #complexMethod:.	! !

!STracerCoreTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:37:37.169'!
testUnaryMethodTrace

	| nodes |
	self tracer traceTest: SSBTest method: #testUnary.
	nodes := self callgraphGetNodesFor: SSBClassA method: #unaryMethod.
	self assert: nodes size = 1.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testUnary.! !

STracerTest subclass: #STracerInitializeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:35.514'!
testCreateCallNode

	| nodes |
	self tracer createNewCallgraphOn: SSBTest with: #testDoNothing.
	self assert: (self tracer 
				createNewCallNodeOn: SSBClassB with: #compute receiverType: SSBClassB callsBefore: 0) = 1.
	nodes := self callgraphGetNodesFor: SSBClassB method: #compute.
	self assert: nodes notEmpty.
	self assert: nodes first fromSender clientClass == SSBTest.
	self assert: nodes first fromSender clientSelector == #testDoNothing.
	! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:46.602'!
testCreateNewCallGraph

	self assert: self rootNode isNil.
	self tracer createNewCallgraphOn: SSBTest with: #testDoNothing.
	self assert: self tracer currentId = self tracer rootId.
	self assert: self rootNode notNil.
	self assert: self rootNode nextCalls isEmpty.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:44.899'!
testCreateRootNode

	self assert: self tracer currentId = -1.
	self tracer createRootNode: SSBTest withTest: #testDoNothing.
	self assert: self tracer currentId = self tracer rootId.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:04.328'!
testCreateWrapper

	| wrappers |
	wrappers := OrderedCollection new.
	[self tracer createWrapperOn: SSBClassA and: #doNothing addTo: wrappers.
	self assert: wrappers size = 1.
	self assert: wrappers first tracer == self tracer.
	self assert: wrappers first wrappedClass == SSBClassA.
	self assert: wrappers first clientSelector == #doNothing.] 
		ensure: [wrappers first uninstall].! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'tfel 3/6/2012 15:28:40.032'!
testCreateWrappers

	| wrappers |
	[wrappers := self tracer createWrappers.
	self assert: wrappers notEmpty.
	self assert: wrappers size > 1.
	wrappers do: [:each | 
		each wrappedClass isMeta
			ifFalse: [self assert: (each wrappedClass category beginsWith: 'Path-Tests-Sandbox')]]
	] ensure: [self tracer removeWrappers: wrappers].! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:51.842'!
testResetCallGraph

	self tracer createNewCallgraphOn: SSBTest with: #testDoNothing.
	self tracer resetCallgraph.
	self assert: self tracer currentId = -1.
	self assert: self rootNode isNil.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:55'!
testRootId

	self assert: self tracer rootId = 0.
	self deny: self tracer rootId = self tracer currentId.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:50.325'!
testRunTestWithoutInstrumentation

	self tracer createNewCallgraphOn: SSBTest with: #testDoNothing.
	self tracer runRootNode.
	self assert: self rootNode nextCalls isEmpty.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 5/2/2010 12:09'!
testRunTestWithoutInstrumentationAndChoosenTest

	self should: [self tracer runRootNode] raise: MessageNotUnderstood.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'tfel 3/6/2012 15:28:30.36'!
testSelectedCategories

	self assert: SProject current categories size = 1.
	self assert: (SProject current categories includes: #'Path-Tests-Sandbox').! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:34:48.403'!
testTestCaseIsAvailable

	self assert: (SProject current categories includes: SSBTest category)! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:40'!
testTracerInitialActiveState

	self deny: self tracer active.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:45'!
testTracerInitialCallgraphModel

	self assert: self tracer callgraphModel notNil.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:53:59.223'!
testTracerInitialCategories

	self assert: self tracer selectedCategories = SProject current categories.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:53'!
testTracerInitialCurrentId

	self assert: self tracer currentId = -1.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:40'!
testTracerInitialProfilingMode

	self assert: self tracer profilingMode = #normal.! !

!STracerInitializeTest methodsFor: 'testing' stamp: 'mp 3/5/2010 17:40'!
testTracerInitialViewMode

	self assert: self tracer viewMode = #classes.! !

STracerTest subclass: #STracerProfilingTraceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

!STracerProfilingTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:21:43.97'!
testGetCallWrapperClass

	self tracer profilingMode: #%.
	self assert: self tracer getCallWrapperClass == SProfilingWrapper.	
	self tracer profilingMode: #counts.
	self assert: self tracer getCallWrapperClass == SCallWrapper.
	self tracer profilingMode: #ms.
	self assert: self tracer getCallWrapperClass == SProfilingWrapper.	
	self tracer viewMode: #identities.
	self tracer profilingMode: #counts.
	self assert: self tracer getCallWrapperClass == SViewWrapper.		! !

!STracerProfilingTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:10.401'!
testProfilingAndView

	| nodes |
	self tracer profilingMode: #%.
	self tracer viewMode: #objects.
	self tracer traceTest: SSBTest method: #testDelay.
	nodes := self callgraphGetNodesFor: SSBClassA method: #delayedMethod.
	self assert: nodes first profiles notEmpty.
	self assert: nodes first views notEmpty.
	self assert: (nodes first profiles at: #%) asInteger > 60.
	self assert: (nodes first views at: #classes) = SSBClassA.
	self assert: (nodes first views at: #objects) = '0'.
	self assert: nodes first asString = ('[', (nodes first profiles at: #%),']', 
								(nodes first views at: #objects), '>>delayedMethod').! !

!STracerProfilingTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:21.284'!
testProfilingCounts

	| nodes |
	self tracer profilingMode: #counts.
	self tracer traceTest: SSBTest method: #testRecursion.
	nodes := self callgraphGetNodesFor: SSBClassA method: #recursionMethod:.
	nodes withIndexDo: [:each :index | 
		self assert: each profiles notEmpty.
		self assert: (each profiles at: #counts) asInteger = index.		
		self assert: (each asString beginsWith: '[', (each profiles at: #counts),']SSBClassA>>recursionMethod:')].
! !

!STracerProfilingTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:16.268'!
testProfilingMilliSeconds

	| nodes |
	self tracer profilingMode: #ms.
	self tracer traceTest: SSBTest method: #testDelay.
	nodes := self callgraphGetNodesFor: SSBClassA method: #delayedMethod.
	self assert: nodes first profiles notEmpty.
	self assert: (nodes first profiles at: #ms) asInteger >= 95.
	self assert: nodes first asString = ('[', (nodes first profiles at: #ms),']SSBClassA>>delayedMethod').! !

!STracerProfilingTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:26.185'!
testProfilingPercentage
	
	| nodes |
	self tracer profilingMode: #%.
	self tracer traceTest: SSBTest method: #testDelay.
	nodes := self callgraphGetNodesFor: SSBClassA method: #delayedMethod.
	self assert: nodes first profiles notEmpty.
	self assert: (nodes first profiles at: #%) asInteger > 60.
	self assert: nodes first asString = ('[', (nodes first profiles at: #%),']SSBClassA>>delayedMethod').! !

!STracerProfilingTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:30.564'!
testWithoutProfiling

	| nodes |
	self tracer profilingMode: #normal.
	self tracer traceTest: SSBTest method: #testDelay.
	nodes := self callgraphGetNodesFor: SSBClassA method: #delayedMethod.
	self assert: nodes first profiles isEmpty.
	self assert: nodes first asString = 'SSBClassA>>delayedMethod'.! !

STracerTest subclass: #STracerRefinementTraceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:50.348'!
testCreateExploreWrapper

	| nodes wrapper |
	self tracer traceTest: SSBTest method: #testDoNothing.
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	wrapper := self tracer createExploreWrapper: #selfBefore on: nodes first.
	self assert: wrapper notNil.
	self assert: wrapper objects isEmpty.
	self assert: wrapper wrappedClass == SSBClassA.
	self assert: wrapper clientSelector == #doNothing.
	self assert: wrapper numberOfCallsBefore = 0.
	self assert: wrapper exploreWhat == #selfBefore.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:18.981'!
testExploreArgumentsBinary

	| obj args |
	self tracer traceTest: SSBTest method: #testBinary.
	obj := self tracer 
				explore: #arguments 
				on: (self callgraphGetNodesFor: SSBClassA method: #+) first.
	args := obj first.
	self assert: args notEmpty.
	self assert: args size = 1.
	self assert: args first = 5.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:57.327'!
testExploreArgumentsMultiply

	| obj args |
	self tracer traceTest: SSBTest method: #testParamMethod.
	obj := self tracer 
				explore: #arguments 
				on: (self callgraphGetNodesFor: SSBClassA method: #param:method:) first.
	args := obj first.
	self assert: args notEmpty.
	self assert: args size = 2.
	self assert: args = #(1 2).! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:38.029'!
testExploreArgumentsUnary

	| args |
	self tracer traceTest: SSBTest method: #testUnary.
	args := self tracer 
				explore: #arguments 
				on: (self callgraphGetNodesFor: SSBClassA method: #unaryMethod) first.
	self assert: args first isEmpty.
	! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:02.364'!
testExploreExplicitReturn

	| return |
	self tracer traceTest: SSBTest method: #testUnary.
	return := self tracer 
				explore: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #unaryMethod) first.
	self assert: return notEmpty.
	self assert: return first.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:17.204'!
testExploreOn

	| obj |
	self tracer traceTest: SSBTest method: #testDoNothing.
	 obj := self tracer explore: #selfBefore on: (self callgraphGetNodesFor: SSBClassA method: #doNothing) first.
	self assert: obj notEmpty.
	self assert: obj size = 1.
	self assert: obj first myValue = 0.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:45.538'!
testExploreRecursionMethodFirst

	| obj |
	self tracer traceTest: SSBTest method: #testRecursion.
	obj := self tracer 
				explore: #arguments 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) first.
	self assert: obj first first = 5.	
			
	obj := self tracer 
				explore: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) first.	
	self assert: obj first = 15.	! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:00.292'!
testExploreRecursionMethodLast

	| obj |
	self tracer traceTest: SSBTest method: #testRecursion.
	obj := self tracer 
				explore: #arguments 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) last.
	self assert: obj first first = 0.	
			
	obj := self tracer 
				explore: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) last.	
	self assert: obj first = 0.	! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:14.23'!
testExploreRecursionMethodMiddle

	| obj |
	self tracer traceTest: SSBTest method: #testRecursion.
	obj := self tracer 
				explore: #arguments 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) third.
	self assert: obj first first = 3.	
			
	obj := self tracer 
				explore: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) third.	
	self assert: obj first = 6.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:40.934'!
testExploreSelfAfter

	| obj |
	self tracer traceTest: SSBTest method: #testModelView.
	 obj := self tracer explore: #selfAfter on: (self callgraphGetNodesFor: SSBClassB method: #myValue:) first.
	self assert: obj notEmpty.
	self assert: obj size = 1.
	self assert: obj first class = SSBClassB.
	self assert: obj first myValue = 2.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:45.163'!
testExploreSelfBefore

	| obj |
	self tracer traceTest: SSBTest method: #testModelView.
	 obj := self tracer explore: #selfBefore on: (self callgraphGetNodesFor: SSBClassB method: #myValue:) first.
	self assert: obj notEmpty.
	self assert: obj size = 1.
	self assert: obj first class = SSBClassB.
	self assert: obj first myValue = nil.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:36:37.393'!
testExploreSelfBeforeAndAfterDifference

	| before after node |
	self tracer traceTest: SSBTest method: #testModelView.
	node := (self callgraphGetNodesFor: SSBClassB method: #myValue:) first.
	before := (self tracer explore: #selfBefore on: node) first.
	after := (self tracer explore: #selfAfter on: node) first.
	self deny: before == after.
	self assert: before myValue isNil.
	self assert: after myValue = 2.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:34.724'!
testExploreSelfReturn

	| obj |
	self tracer traceTest: SSBTest method: #testDoNothing.
	obj := self tracer explore: #return on: (self callgraphGetNodesFor: SSBClassA method: #doNothing) first.
	self assert: obj notEmpty.
	self assert: obj size = 1.
	self assert: obj first class == SSBClassA.
	self assert: obj first myValue = 0.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:10.635'!
testExploreTypes

	|  types |
	self tracer traceTest: SSBTest method: #testParamMethod.
	types := self tracer 
				explore: #types 
				on: (self callgraphGetNodesFor: SSBClassA method: #param:method:) first.
	self assert: types notEmpty.
	self assert: types size = 3.

	self assert: types first key == #arguments.
	self assert: types first value size = 2.
	self assert: types first value first = SmallInteger.
	self assert: types first value second = SmallInteger.

	self assert: types second key == #self.
	self assert: types second value == SSBClassA.

	self assert: types third key == #result.
	self assert: types third value == Array.	! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:41.333'!
testNumberOfCallsBeforeMethodChain

	| wrapper |
	self tracer traceTest: SSBTest method: #testMethodChain.
	wrapper := self tracer 
				createExploreWrapper: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #methodChain:) first.
	self assert: wrapper numberOfCallsBefore = 0.
	
	wrapper := self tracer 
				createExploreWrapper: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #methodChain:) second.
	self assert: wrapper numberOfCallsBefore = 1.
	
	wrapper := self tracer 
				createExploreWrapper: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #methodChain:) third.
	self assert: wrapper numberOfCallsBefore = 2.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:38:54.104'!
testNumberOfCallsBeforeRecursion

	| wrapper |
	self tracer traceTest: SSBTest method: #testRecursion.
	wrapper := self tracer 
				createExploreWrapper: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) first.
	self assert: wrapper numberOfCallsBefore = 0.
	
	wrapper := self tracer 
				createExploreWrapper: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) last.
	self assert: wrapper numberOfCallsBefore = 5.
	
	wrapper := self tracer 
				createExploreWrapper: #return 
				on: (self callgraphGetNodesFor: SSBClassA method: #recursionMethod:) third.
	self assert: wrapper numberOfCallsBefore = 2.! !

!STracerRefinementTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:05.983'!
testRunOwnExploreWrapper

	| nodes wrapper a |
	self tracer traceTest: SSBTest method: #testDoNothing.
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	[wrapper := self tracer createExploreWrapper: #selfBefore on: nodes first.
	wrapper install.
	a := SSBClassA new myValue: 123.
	a doNothing.
	self assert: wrapper objects notEmpty.
	self assert: wrapper objects size = 1.
	self assert: wrapper objects first myValue = a myValue.
	self deny: wrapper objects first == a]
		ensure: [wrapper uninstall].! !

!STracerTest methodsFor: 'helper' stamp: 'mp 3/8/2010 16:23'!
callgraphGetNodesFor: aClass method: aMethodSelector
	| nodes |
	nodes := OrderedCollection new.
	self tracer callgraphModel traverseCallGraph: self rootNode 
			with: [:aNode | 
				(aNode clientClass == aClass and: [aNode clientSelector == aMethodSelector])
					ifTrue: [nodes add: aNode]].
	^ nodes! !

!STracerTest methodsFor: 'helper' stamp: 'mp 3/8/2010 16:22'!
rootNode

	^ self tracer callgraphModel rootNode! !

!STracerTest methodsFor: 'testing' stamp: 'mp 3/27/2012 16:12:54.2'!
setUp

	oldCurrentProject := SProject current.
	SProject setCurrent: #sandbox.
	self tracer: (SCallgraphTracer on: SProject current).
	self tracer callgraphModel: (SCallgraphModel on: self tracer).! !

!STracerTest methodsFor: 'testing' stamp: 'mp 7/7/2010 16:53:51.492'!
tearDown

	SProject setCurrentProject: oldCurrentProject.! !

!STracerTest methodsFor: 'accessing' stamp: 'mp 3/5/2010 17:48'!
tracer
	"Answer the value of tracer"

	^ tracer! !

!STracerTest methodsFor: 'accessing' stamp: 'mp 3/5/2010 17:48'!
tracer: anObject
	"Set the value of tracer"

	tracer := anObject! !

STracerTest subclass: #STracerViewTraceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

!STracerViewTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:38.006'!
testClassesView

	| nodes |
	self tracer traceTest: SSBTest method: #testDoNothing.
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	self assert: (nodes first views at: #classes) = SSBClassA.
	self assert: nodes first asString = 'SSBClassA>>doNothing'.! !

!STracerViewTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:21:48.899'!
testGetCallWrapperClass

	self tracer viewMode: #objects.
	self assert: self tracer getCallWrapperClass == SViewWrapper.
	self tracer viewMode: #classes.
	self assert: self tracer getCallWrapperClass == SCallWrapper.	
	self tracer viewMode: #identities.
	self assert: self tracer getCallWrapperClass == SViewWrapper.
	self tracer profilingMode: #%.
	self assert: self tracer getCallWrapperClass == SProfilingWrapper.	
	self tracer profilingMode: #counts.
	self assert: self tracer getCallWrapperClass == SViewWrapper.! !

!STracerViewTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:32.175'!
testIdentityView

	| nodes |
	self tracer viewMode: #identities.
	self tracer traceTest: SSBTest method: #testDoNothing.
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	self assert: (nodes first views at: #identities) asInteger > 0.
	self assert: (nodes first asString beginsWith: 'SSBClassA').
	self assert: (nodes first asString occurrencesOf: $() = 1.
	self assert: ((nodes first asString subStrings: '(') second subStrings: ')') first asInteger > 0.! !

!STracerViewTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:35.024'!
testObjectsView

	| nodes |
	self tracer viewMode: #objects.
	self tracer traceTest: SSBTest method: #testDoNothing.
	nodes := self callgraphGetNodesFor: SSBClassA method: #doNothing.
	self assert: (nodes first views at: #objects) = '0'.
	self assert: nodes first asString = '0>>doNothing'.! !

!STracerViewTraceTest methodsFor: 'testing' stamp: 'mp 7/7/2010 17:39:22.658'!
testPrintString

	| a |
	a := SSBClassA new.
	self assert: a asString = '0'.
	a myValue: 123.
	self assert: a asString = '123'.
	self assert: SSBClassB new asString = 'a SSBClassB'.! !

TestCase subclass: #SWrapperTest
	instanceVariableNames: 'myObject originalMethod methodRef'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

SWrapperTest subclass: #SAroundWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

!SAroundWrapperTest methodsFor: '-- all --' stamp: 'mp 7/7/2010 17:45:09.713'!
tearDown

	SAroundWrapperMethodOne nuke.
	SAroundWrapperMethodTwo nuke.
	super tearDown.! !

!SAroundWrapperTest methodsFor: '-- all --' stamp: 'mp 7/7/2010 17:45:13.183'!
testAround

	| wrapper |
	wrapper := SAroundWrapperMethodOne on: #methodOne inClass: SClassA.
	self assert: self myObject methodOne.
	wrapper install.
	self deny: self myObject methodOne.
	wrapper uninstall.
	self assert: self myObject methodOne.
	
	! !

!SAroundWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:59.159'!
testProceed

	| wrapper1 wrapper2 |
	wrapper1 := SAroundWrapperMethodTwo on: #methodTwo: inClass: SClassA.
	wrapper2 := SAroundWrapperMethodTwo on: #methodTwo: inClass: SClassA.	
	self assert: (self myObject methodTwo: 1) = 1.
	wrapper1 install.
	self assert: (self myObject methodTwo: 1) = 3.
	wrapper1 uninstall.
	self assert: (self myObject methodTwo: 1) = 1.
	wrapper2 install.
	self assert: (self myObject methodTwo: 1) = 3.
	wrapper2 uninstall.! !

SWrapperTest subclass: #SBeforeWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

!SBeforeWrapperTest methodsFor: 'private' stamp: 'mp 1/22/2010 15:49'!
checkTouchs:  aWrapperAssocCollection

	aWrapperAssocCollection do: [:each | self assert: each key touched = each value].
	! !

!SBeforeWrapperTest methodsFor: '-- all --' stamp: 'mp 7/7/2010 17:44:16.005'!
tearDown

	SBeforeWrapper nuke.
	super tearDown.! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:12.745'!
testAccessCompiledMethod

	| wrappers |
	wrappers := OrderedCollection new.
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers first install.
	wrappers  second install.
	wrappers third install.
	self checkWrapperChain: #(Wrapper Wrapper Wrapper Original) of: self methodRef.
	self assert: wrappers third unwrappedMethod = originalMethod.! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:22.331'!
testInstallationMultipleWrappers

	| wrappers |
	wrappers := OrderedCollection new.
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	self checkWrapperChain: #(Original) of: self methodRef.
	wrappers first install.
	self checkWrapperChain: #(Wrapper Original) of: self methodRef.
	wrappers  second install.
	self checkWrapperChain: #(Wrapper Wrapper Original) of: self methodRef.
	self assert: self methodRef compiledMethod == wrappers second.
	wrappers third install.
	self checkWrapperChain: #(Wrapper Wrapper Wrapper Original) of: self methodRef.
	self assert: self methodRef compiledMethod == wrappers third.
	self assert: self methodRef compiledMethod clientMethod == wrappers second.
	self assert: self methodRef compiledMethod clientMethod clientMethod  == wrappers first.
	wrappers first uninstall.
	self checkWrapperChain: #(Wrapper Wrapper Original) of: self methodRef.
	self assert: self methodRef compiledMethod == wrappers third.
	self assert: self methodRef compiledMethod clientMethod  == wrappers second.
	wrappers third uninstall.
	self checkWrapperChain: #( Wrapper Original) of: self methodRef.
	self assert: self methodRef compiledMethod == wrappers second.
	wrappers second uninstall.
	self checkWrapperChain: #( Original) of: self methodRef.! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:27.061'!
testInstallationOneWrapper

	| wrapper |
	wrapper := SBeforeWrapper on: #methodOne inClass: SClassA.
	self deny: self methodRef compiledMethod isMwMethodWrapper.
	wrapper install.
	self assert: self methodRef compiledMethod isMwMethodWrapper.
	wrapper uninstall.
	self deny: self methodRef compiledMethod isMwMethodWrapper.! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:18.542'!
testOneWrapper

	| wrapper |
	wrapper := SBeforeWrapper on: #methodOne inClass: SClassA.
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper -> 0}. 
	wrapper install.
	self checkTouchs: {wrapper -> 0}. 
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper -> 1}. 
	wrapper uninstall.
	self assert: ((MethodReference class: SClassA selector: #methodOne) compiledMethod isKindOf: CompiledMethod).
	self checkTouchs: {wrapper -> 1}. 
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper -> 1}.! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:09.372'!
testTwoWrappers

	| wrapper1 wrapper2 |
	wrapper1 := SBeforeWrapper on: #methodOne inClass: SClassA.
	wrapper2 := SBeforeWrapper on: #methodOne inClass: SClassA.
	self checkTouchs: {wrapper1 -> 0. wrapper2 -> 0}. 
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper1 -> 0. wrapper2 -> 0}. 	
	wrapper1 install.
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper1 -> 1. wrapper2 -> 0}. 
	wrapper2 install.
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper1 -> 2. wrapper2 -> 1}. 	
	wrapper2 uninstall.
	self assert: self methodRef compiledMethod isMwMethodWrapper.
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper1 -> 3. wrapper2 -> 1}. 
	wrapper1 uninstall.
	self deny:  self methodRef compiledMethod isMwMethodWrapper.
	self assert: self myObject methodOne.
	self checkTouchs: {wrapper1 -> 3. wrapper2 -> 1}. 	! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 1/22/2010 17:02'!
testWithoutWrapper

	self assert: self myObject methodOne.! !

!SBeforeWrapperTest methodsFor: 'tests' stamp: 'mp 7/7/2010 17:44:30.095'!
testWrappingWrapper

	| wrappers |
	wrappers := OrderedCollection new.
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers add: (SBeforeWrapper on: #methodOne inClass: SClassA).
	wrappers first install.
	wrappers  second install.
	wrappers third install.
	self checkWrapperChain: #(Wrapper Wrapper Wrapper Original) of: self methodRef.
	self assert: wrappers third wrappingWrapper = nil.
	self assert: wrappers second wrappingWrapper = wrappers third.
	self assert: wrappers first wrappingWrapper = wrappers second.
	wrappers second uninstall.
	self assert: wrappers second wrappingWrapper = nil.
	self assert: wrappers first wrappingWrapper = wrappers third.! !

!SWrapperTest methodsFor: 'private' stamp: 'mp 3/5/2010 12:57'!
checkWrapperChain: anArray of: aMethodRef

	| item |
	item := aMethodRef compiledMethod.
	anArray do: [:each |
		each == #Wrapper
			ifTrue: [	self assert: item isMwMethodWrapper.
					item := item clientMethod].
		each == #Original
			ifTrue: [self deny: item isMwMethodWrapper]].! !

!SWrapperTest methodsFor: 'accessing' stamp: 'mp 1/22/2010 16:54'!
methodRef
	"Answer the value of methodRef"

	^ methodRef! !

!SWrapperTest methodsFor: 'accessing' stamp: 'mp 1/22/2010 16:54'!
methodRef: anObject
	"Set the value of methodRef"

	methodRef := anObject! !

!SWrapperTest methodsFor: 'accessing' stamp: 'mp 1/22/2010 16:55'!
myObject
	"Answer the value of myObject"

	^ myObject! !

!SWrapperTest methodsFor: 'accessing' stamp: 'mp 1/22/2010 16:55'!
myObject: anObject
	"Set the value of myObject"

	myObject := anObject! !

!SWrapperTest methodsFor: '-- all --' stamp: 'mp 7/7/2010 17:43:07.462'!
setUp

	self myObject: SClassA new.
	originalMethod := (MethodReference class: SClassA selector: #methodOne) compiledMethod.
	self methodRef: (MethodReference class: SClassA selector: #methodOne).! !

!SWrapperTest methodsFor: '-- all --' stamp: 'mp 1/22/2010 16:56'!
tearDown

	self methodRef actualClass methodDictionary at: self methodRef methodSymbol put: originalMethod.! !

TestCase subclass: #THEmbeddedBlockMethodDescriptorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester'!

!THEmbeddedBlockMethodDescriptorTest methodsFor: 'private' stamp: 'mha 5/25/2010 14:23'!
doNotChangeOrInvokeThisMethod
	| gunk plink |
	[ :each | [ :beach | ] value: plink ] value: gunk.
	^ [ :bruce :bam | bruce perform: bam ] value: gunk value: plink! !

!THEmbeddedBlockMethodDescriptorTest methodsFor: 'testing' stamp: 'mha 9/23/2010 10:50'!
testDescriptorEquality
	| m d1 d2 |
	m := THEmbeddedBlockMethodDescriptorTest >> #doNotChangeOrInvokeThisMethod.
	d1 := THEmbeddedBlockMethodDescriptor in: m startingAt: 26 nlr: false.
	d2 := THEmbeddedBlockMethodDescriptor in: m startingAt: 26 nlr: false.
	self
		assert: d1 = d2;
		assert: d1 hash = d2 hash;
		assert: d1 = d1! !

!THEmbeddedBlockMethodDescriptorTest methodsFor: 'testing' stamp: 'mha 9/23/2010 10:51'!
testInferProperties
	| m desc dict blockStart blockEnd |
	m := THSBClassA >> #collectCollection:.
	dict := m startpcsToBlockExtents.
	
	blockStart := dict keys detect: [ :k | k ~= m initialPC ].
	blockEnd := blockStart + ((m at: blockStart - 2) * 256) + (m at: blockStart - 1) - 1.
	self
		assert: blockStart notNil;
		assert: blockEnd > blockStart.
	
	desc := THEmbeddedBlockMethodDescriptor in: m startingAt: blockStart nlr: false.
	self
		assert: desc embeddingMethod = m;
		assert: desc startPC = blockStart;
		assert: desc endPC = blockEnd;
		assert: desc numArgs = 1;
		assert: desc numCopied = 1;
		assert: desc numLocals = 2;
		assert: desc numTemps = 4! !

!THEmbeddedBlockMethodDescriptorTest methodsFor: 'testing' stamp: 'mha 9/23/2010 10:51'!
testMultipleDescriptors
	| m d1 d2 d3 |
	m := THEmbeddedBlockMethodDescriptorTest >> #doNotChangeOrInvokeThisMethod.
	d1 := THEmbeddedBlockMethodDescriptor in: m startingAt: 26 nlr: false.
	d2 := THEmbeddedBlockMethodDescriptor in: m startingAt: 30 nlr: false.
	d3 := THEmbeddedBlockMethodDescriptor in: m startingAt: 42 nlr: false.
	self
		assert: d1 endPC = 34;
		assert: d1 numCopied = 1;
		assert: d1 numArgs = 1;
		assert: d1 numLocals = 0;
		assert: d2 endPC = 31;
		assert: d2 numCopied = 0;
		assert: d2 numArgs = 1;
		assert: d2 numLocals = 0;
		assert: d3 endPC = 45;
		assert: d3 numCopied = 0;
		assert: d3 numArgs = 2;
		assert: d3 numLocals = 0
		! !

!THEmbeddedBlockMethodDescriptorTest methodsFor: 'testing' stamp: 'mha 9/23/2010 10:51'!
testSortDescriptors
	| m d1 d2 d3 sc |
	m := THEmbeddedBlockMethodDescriptorTest >> #doNotChangeOrInvokeThisMethod.
	d1 := THEmbeddedBlockMethodDescriptor in: m startingAt: 26 nlr: false.
	d2 := THEmbeddedBlockMethodDescriptor in: m startingAt: 30 nlr: false.
	d3 := THEmbeddedBlockMethodDescriptor in: m startingAt: 42 nlr: false.
	sc := SortedCollection newFrom: { d2. d1. d3 }.
	self
		assert: sc first = d3;
		assert: sc second = d2;
		assert: sc third = d1! !

TestCase subclass: #THSBSandboxTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester-Sandbox'!

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 10:54:50.473'!
testArrayWithObject
	| obj |
	obj := THSBClassA new.
	obj b: {'String'. 1}! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:13:43.828'!
testAssociation
	| obj |
	obj := THSBClassA new.
	obj b: (#key -> 'value').! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 17:45:41.916'!
testAssociationChange
	| obj |
	obj := THSBClassA new.
	obj b: (#key -> 'value').
	obj b: (1 -> 2).
	! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:14:29.021'!
testAssociationChangeElement
	| obj |
	obj := THSBClassA new.
	obj b: (#key -> 'value').
	obj b: (#otherKey -> 2).! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:14:40.391'!
testAssociationChangeKey
	| obj |
	obj := THSBClassA new.
	obj b: (#key -> 'value').
	obj b: (1 -> '2').! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 10:55:47.944'!
testBagWithNumbers
	| obj |
	obj := THSBClassA new.
	obj b: (Bag with: 1 with: 1 with: 3.0)! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:22:31.81'!
testChangeFromArrayToOrderedCollection
	| obj |
	obj := THSBClassA new.
	obj b: {1. 2}.
	obj b: obj b asOrderedCollection.! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:23:27.719'!
testChangeFromSingleObjectToCollection
	| obj |
	obj := THSBClassA new.
	obj b: 1.
	obj b: (OrderedCollection with: obj b with: 2).! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 16:19'!
testCollectCollection
	| jm |
	jm := THSBClassA new.
	self
		assert: (jm collectCollection: #( 1 2 3 )) = #( true true true )! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/12/2011 14:39:33.268'!
testConstructor
	| obj |
	obj := THSBClassA a: 1 b: 2 c: 3.
	self assert: obj a = 1.
	self assert: obj b = 2.
	self assert: obj c = 3.		! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 17:28:25.724'!
testDictionary
	| obj |
	obj := THSBClassA new.
	obj b: (Dictionary newFrom: {#a->1. #b->2.}).! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/3/2011 15:04:16.275'!
testDictionaryChange
	| obj |
	obj := THSBClassA new.
	obj b: Dictionary new.
	obj b add: #a -> 1.
"	obj b: obj b."
	obj b at: #a put: 2.0.
"	obj b: obj b."! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/27/2010 17:38'!
testGetAccessor

	self assert: THSBClassA new a = 1.! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/3/2011 09:39:33.498'!
testIdentitySet
	| obj |
	obj := THSBClassA new.
	obj b: IdentitySet new.
	obj b add: $a.
	obj b add: $a.
	obj b: obj b.! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/3/2011 09:47:36.046'!
testIncrementalAddOfNumbers
	| obj |
	obj := THSBClassA new.
	obj b: OrderedCollection new.
	obj b 
			add: 1;
			add: -2;
			addLast: 2.0;
			add: (3/5).
"	obj b: obj b."! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:15:43.266'!
testInterval
	| obj |
	obj := THSBClassA new.
	obj b: (Interval from: 1 to: 10).! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 16:19'!
testLocals
	| jm |
	jm := THSBClassA new.
	self
		assert: (jm locals: 23 which: 1) = 23;
		assert: (jm locals: 42.0 which: 1) = 42.0;
		assert: (jm locals: 'yo' which: 2) = 'yo';
		assert: (jm locals: #yo which: 2) = #yo;
		assert: (jm locals: nil which: 3) = nil;
		assert: (jm locals: Smalltalk which: 3) = Smalltalk! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 17:18'!
testLoopBlock

	| number |
	number := 1.
	2 timesRepeat: [number := THSBClassA new addValueToA: number].
	self assert: number = 3.	! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:18:22.327'!
testMatrix
	| obj |
	obj := THSBClassA new.
	obj b: (Matrix rows: 5 columns: 5 tabulate: [:row :column | row * column]).! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 16:16'!
testMembers
	| sm |
	sm := THSBClassA new.
	self
		assert: (sm a: 23; a) = 23;
		assert: (sm a: 42.0; a) = 42.0;
		assert: (sm b: 'yo'; b) = 'yo';
		assert: (sm b: #yo; b) = #yo;
		assert: (sm c: nil; c) = nil;
		assert: (sm c: Smalltalk; c) = Smalltalk! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/3/2011 15:11:33.839'!
testMethodWithContainersAllOver
	| obj |
	obj := THSBClassA new.
	obj 
		assoc: #a -> 1 
		collection: (OrderedCollection with: 1 with: 'string') 
		dict: (Dictionary newFrom: {#a -> 1. #b -> 2})
		number: 2.! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/3/2011 15:23:18.636'!
testMethodWithContainersAllOverReally
	| obj |
	obj := THSBClassA new.
	obj 
		assoc: #a -> 1 
		collection: (OrderedCollection with: 1 with: 'string') 
		dict: (Dictionary newFrom: {#a -> 1. #b -> 2})
		number: {2}.! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 16:20'!
testNumbers
	| jm |
	jm := THSBClassA new.
	self
		assert: (jm numbers: 23) = 23;
		assert: (jm numbers: 42.23) = 42.23! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 10:54:18.425'!
testOrderedCollectionWithNumber
	| obj |
	obj := THSBClassA new.
	obj b: (OrderedCollection with: 1 with: 2.0)! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 10:53:48.814'!
testOrderedCollectionWithSmallInteger
	| obj |
	obj := THSBClassA new.
	obj b: (OrderedCollection with: 1 with: 2)! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 14:05:17.636'!
testRecursiveCollections
	| obj |
	obj := THSBClassA new.
	obj b: (OrderedCollection with: {1. 2.} with: {$a. $b} with: (Set with: 'string')).! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 17:03'!
testReturn

	self assert: (THSBClassA new numbers: 1 ) = 1
	! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 10:55:23.458'!
testSetWithIntegers
	| obj |
	obj := THSBClassA new.
	obj b: (Set with: 1 with: 2 with: 3)! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/2/2011 11:16:01.643'!
testString
	| obj |
	obj := THSBClassA new.
	obj b: 'string'.! !

!THSBSandboxTest methodsFor: 'testing-container' stamp: 'mp 5/3/2011 14:50:27.431'!
testStringAndSymbol
	| obj |
	obj := THSBClassA new.
	obj b: 'string'.
	obj b: #string! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 16:20'!
testStringsMagnitudesAndObjects
	| jm |
	jm := THSBClassA new.
	self
		assert: (jm strings: 'hi' magnitudes: $E andObjects: 23 index: 1) = 'hi';
		assert: (jm strings: #ho magnitudes: #eep -> #oop andObjects: 'xyz' index: 1) = #ho;
		assert: (jm strings: #'a b c' magnitudes: DateAndTime current andObjects: nil index: 3) = nil;
		assert: (jm strings: 'Yo!!' magnitudes: TimeStamp now andObjects: self index: 3) = self! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/12/2011 14:42:47.9'!
testSuper

	self assert: THSBClassSubB2 new mark = '?'.! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 17:41'!
testTraceMethodExecution

	self assert: (THSBClassA new sayHelloTo: 'world') = 'Hello world!!'
	! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 17:53'!
testTraceMethodExecutionWithBlock

	self assert: (THSBClassA new concatTwoStrings: [:str1 :str2 | str1, ' ', str2, '!!']) = 'Hello world!!'
	! !

!THSBSandboxTest methodsFor: 'testing' stamp: 'mp 5/26/2010 17:47'!
testTraceMethodExecutionWithCondition

	| str |
	'1' = 1 asString
		ifTrue: [str := THSBClassA new sayHelloTo: 'world']
		ifFalse: [str := 'Germany'].
	self assert: str = 'Hello world!!'.
	! !

TestCase subclass: #THTest
	instanceVariableNames: 'harvester'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester'!

THTest subclass: #THHeisenbugTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester'!

!THHeisenbugTest methodsFor: 'checking' stamp: 'mp 3/17/2010 12:10'!
checkTestResult: aTestResult

	self assert: aTestResult tests size = 1.
	self assert: aTestResult hasPassed.
	self deny: aTestResult hasFailures.
	self deny: aTestResult hasErrors.! !

!THHeisenbugTest methodsFor: 'running' stamp: 'mp 5/26/2010 10:56'!
setUp

	super setUp.
	self harvester eager.! !

!THHeisenbugTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:04:49.358'!
testGetAccessor

	self checkTestResult: (self harvester 
							harvest: [THSBSandboxTest 
										runWithoutTimeout: #testGetAccessor]).! !

!THHeisenbugTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:04:54.734'!
testLoopBlock

	self checkTestResult: (self harvester 
							harvest: [THSBSandboxTest runWithoutTimeout: #testLoopBlock]).! !

!THHeisenbugTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:04:59.165'!
testReturn

	self checkTestResult: (self harvester 
							harvest: [THSBSandboxTest runWithoutTimeout: #testReturn]).! !

!THHeisenbugTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:05:03.006'!
testTraceMethodExecution

	self checkTestResult: (self harvester 
							harvest: [THSBSandboxTest 
										runWithoutTimeout: #testTraceMethodExecution]).! !

!THHeisenbugTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:05:08.445'!
testTraceMethodExecutionWithBlock

	self checkTestResult: (self harvester 
							harvest: [THSBSandboxTest 
									runWithoutTimeout: #testTraceMethodExecutionWithBlock]).! !

!THHeisenbugTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:05:14.35'!
testTraceMethodExecutionWithCondition

	self checkTestResult: (self harvester 
							harvest: [THSBSandboxTest 
							runWithoutTimeout: #testTraceMethodExecutionWithCondition]).! !

THTest subclass: #THMonoBucketTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester'!

!THMonoBucketTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 13:28:01.853'!
byteString
	^ self container: ByteString
		elements: Character
		keys: nil! !

!THMonoBucketTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 13:28:12.084'!
byteSymbol
	^ self container: ByteSymbol
		elements: Character
		keys: nil! !

!THMonoBucketTest methodsFor: 'running' stamp: 'mp 5/26/2010 11:09'!
checkTypes: expected
	expected keysAndValuesDo: [ :method :types |
		types keysAndValuesDo: [ :pc :type |
			self assert: type = (self harvester bucket typeAt: pc in: method) ] ]! !

!THMonoBucketTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 13:28:26.198'!
container: aClass elements: elementType keys: keyType
	| container |
	container := THContainerType new.
	container 
		typeOfContainer: aClass;
		typeOfElements: elementType;	
		typeOfKeys: keyType.
	^ container! !

!THMonoBucketTest methodsFor: 'expected types' stamp: 'mp 5/3/2011 15:55:23.638'!
expTypesTestConditional
	^ Dictionary newFrom: {
		THSBSandboxTest >> #testTraceMethodExecutionWithCondition ->
			(Dictionary newFrom: {
				45 -> nil.
				46 -> self byteString.
				47 -> SmallInteger.
				48 -> self byteString.
				49 -> True.
				50 -> nil.
				51 -> THSBClassA class.
				52 -> THSBClassA.
				53 -> self byteString.
				54 -> self byteString.
				55 -> nil.
				56 -> self byteString.
				57 -> nil.
				58 -> nil.
				60 -> self byteString.
				61 -> self byteString.
				62 -> THSBSandboxTest.
				63 -> self byteString.
				64 -> self byteString.
				65 -> True.
				66 -> THSBSandboxTest.
				67 -> self byteString
			}).
		THSBClassB >> #initialize ->
			(Dictionary newFrom: {
				21 -> nil.
				22 -> THSBClassB.
				23 -> self byteString.
				24 -> THSBClassB.
				25 -> nil
			}).
		THSBClassB >> #mark: ->
			(Dictionary newFrom: {
				13 -> self byteString.
				14 -> self byteString.
				15 -> self byteString
			})
	}! !

!THMonoBucketTest methodsFor: 'expected types' stamp: 'mp 5/26/2010 16:50'!
expTypesTestGetAccessor
	^ Dictionary newFrom: {
		THSBSandboxTest >> #testGetAccessor ->
			(Dictionary newFrom: {
				25 -> nil.
				26 -> THSBSandboxTest.
				27 -> THSBClassA class.
				28 -> THSBClassA.
				29 -> SmallInteger.
				30 -> SmallInteger.
				31 -> True.
				32 -> THSBSandboxTest.
				33 -> nil
			}).
		THSBClassA >> #initialize ->
			(Dictionary newFrom: {
				17 -> nil.
				18 -> THSBClassA.
				19 -> SmallInteger.
				20 -> THSBClassA.
				21 -> nil
			}).
		THSBClassA >> #a: ->
			(Dictionary newFrom: {
				13 -> SmallInteger.
				14 -> SmallInteger.
				15 -> SmallInteger
			})
	}! !

!THMonoBucketTest methodsFor: 'as yet unclassified' stamp: 'mp 5/5/2011 17:03:50.16'!
expectedFailures
	^ #( #testDictionaryChange )! !

!THMonoBucketTest methodsFor: 'running' stamp: 'mp 5/26/2010 11:09'!
setUp
	super setUp.
	self harvester eagerLeastSpecific.! !

!THMonoBucketTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 13:39:50.323'!
string
	^ self container: String
		elements: Character
		keys: nil! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 3/20/2013 16:42:22.953'!
testArrayWithObject
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testArrayWithObject].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self container: Array elements: Object keys: nil).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 16:59:13.385'!
testAssociation
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testAssociation].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: Association 
						elements: self byteString 
						keys: self byteSymbol).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 16:59:16.601'!
testAssociationChange
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testAssociationChange].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: Association 
						elements: Object 
						keys: Object)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 16:59:20.714'!
testAssociationChangeElement
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testAssociationChangeElement].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: Association 
						elements: Object 
						keys: self byteSymbol)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 16:59:24.184'!
testAssociationChangeKey
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testAssociationChangeKey].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: Association 
						elements: self byteString 
						keys: Object)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 16:59:31.449'!
testChangeFromArrayToOrderedCollection
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testChangeFromArrayToOrderedCollection].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: SequenceableCollection 
						elements: SmallInteger 
						keys: nil)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 16:59:37.48'!
testChangeFromSingleObjectToCollection
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testChangeFromSingleObjectToCollection].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = Object.! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 16:59:41.321'!
testClosureTemps
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testCollectCollection].	
	self 
		assert: (self harvester bucket 
					parameterTypeAt: 1 in: THSBClassA >> #collectCollection:) = 
						(self container: Array elements: SmallInteger keys: nil)! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 16:59:45.544'!
testConditional

	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testTraceMethodExecutionWithCondition].	
	self checkTypes: self expTypesTestConditional! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:08.28'!
testContainerArguments
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	type := self harvester bucket 
				tempTypeAt: 1 in: THSBClassA>>#assoc:collection:dict:number:.
	self assert: type = (self container: Association elements: SmallInteger keys: self byteSymbol).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:00.695'!
testContainerInstVars
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	type := self harvester bucket memberTypeFor: 2 in: THSBClassA.
	self assert: type = Object.! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:18.615'!
testContainerInstVarsAll
	| type |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testMethodWithContainersAllOverReally].
	type := self harvester bucket memberTypeFor: 2 in: THSBClassA.
	self assert: type = (self 
							container: Object 
							elements: Object 
							keys: self byteSymbol).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:25.527'!
testContainerReturn
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	type := self harvester bucket 
					returnTypeFor: THSBClassA>>#assoc:collection:dict:number:.
	self assert: type = (self 
							container: Array 
							elements: Object 
							keys: nil).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:31.59'!
testContainerReturnAll
	| type |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testMethodWithContainersAllOverReally].
	type := self harvester bucket 
					returnTypeFor: THSBClassA>>#assoc:collection:dict:number:.
	self assert: type = (self 
							container: Array 
							elements: (self 
											container: Object 
											elements: Object 
											keys: self byteSymbol) 
							keys: nil).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:37.815'!
testContainerTemps
	| type |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testMethodWithContainersAllOver].
	type := self harvester bucket 
				tempTypeAt: 5 in: THSBClassA>>#assoc:collection:dict:number:.
	self assert: type = (self container: Association elements: SmallInteger keys: self byteSymbol).! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:46.391'!
testContainerTopOfStack
	| initialPC type |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testMethodWithContainersAllOver].
	initialPC := (THSBClassA>>#assoc:collection:dict:number:) initialPC.
	type := self harvester bucket 
				typeAt: initialPC + 4 - 1 
				in: THSBClassA>>#assoc:collection:dict:number:.
	self assert: type = (self container: OrderedCollection elements: Object keys: nil).! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'tfel 7/8/2011 12:05:54.943'!
testConversionBackAndForth
	| bucket recoveredBucket |
	self harvester harvest: [
		THSBSandboxTest testSelectors 
			do: [:each | THSBSandboxTest runWithoutTimeout: each]].
	bucket := self harvester bucket.
	(bucket instVarNamed: 'blockMaps') removeAll. "blockMaps aren't converted"
	recoveredBucket := bucket convertToSymbols convertToTypes..
	self assert: bucket = bucket copy.
	self deny: bucket = bucket convertToTypes.
	self assert: (bucket instVarNamed: 'methodTypeMaps') = 
					(recoveredBucket instVarNamed: 'methodTypeMaps').
	self assert: (bucket instVarNamed: 'methodTempMaps') = 
					(recoveredBucket instVarNamed: 'methodTempMaps').
	self assert: (bucket instVarNamed: 'methodReturnMaps') = 
					(recoveredBucket instVarNamed: 'methodReturnMaps').
	self assert: (bucket instVarNamed: 'memberTypeMaps') = 
					(recoveredBucket instVarNamed: 'memberTypeMaps').
	self assert: (bucket instVarNamed: 'stackpMaps') = 
					(recoveredBucket instVarNamed: 'stackpMaps').
	self assert: (bucket instVarNamed: 'stackMaps') = 
					(recoveredBucket instVarNamed: 'stackMaps').
	self assert: bucket = recoveredBucket.! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'tfel 7/8/2011 11:51:28.484'!
testConversionToSymbols
	"self convertToSymbols"
	| bucket symbolBucket |
	self harvester harvest: [
		THSBSandboxTest testSelectors 
			do: [:each | THSBSandboxTest runWithoutTimeout: each]].
	bucket := self harvester bucket.
	symbolBucket := bucket convertToSymbols.
	self deny: bucket == symbolBucket.
	self assert: (bucket instVarNamed: 'methodTypeMaps') size = 
					(symbolBucket instVarNamed: 'methodTypeMaps') size.
	self assert: (symbolBucket instVarNamed: 'methodTypeMaps') 
					keys anyOne key isSymbol.
	self assert: (symbolBucket instVarNamed: 'methodTypeMaps') 
					keys anyOne value isSymbol.
	self assert: ((symbolBucket instVarNamed: 'methodTypeMaps') 
					values detect: [:each | each size > 0]) first isString.
					
	self assert: (bucket instVarNamed: 'methodTempMaps') size = 
					(symbolBucket instVarNamed: 'methodTempMaps') size.
					
	self assert: (bucket instVarNamed: 'methodReturnMaps') size = 
					(symbolBucket instVarNamed: 'methodReturnMaps') size.
	self assert: [(symbolBucket instVarNamed: 'methodReturnMaps')
					anyOne isSymbol or: [
						(symbolBucket instVarNamed: 'methodReturnMaps')
							anyOne class = THContainerType]].
					
	self assert: (bucket instVarNamed: 'memberTypeMaps') size = 
					(symbolBucket instVarNamed: 'memberTypeMaps') size.
	self assert: ((symbolBucket instVarNamed: 'memberTypeMaps') at: #THSBClassA) = #(Number Object Object).
	
	self assert: (symbolBucket instVarNamed: 'blockMaps') isEmpty.
	self assert: (bucket instVarNamed: 'stackpMaps') size = 
					(symbolBucket instVarNamed: 'stackpMaps') size.
	self assert: (symbolBucket instVarNamed: 'stackpMaps') values anyOne first isNumber.



	! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 6/26/2011 20:13:40.984'!
testConversionToTypes
	| bucket symbolBucket typesBucket |
	self harvester harvest: [
		THSBSandboxTest testSelectors 
			do: [:each | THSBSandboxTest runWithoutTimeout: each]].
	bucket := self harvester bucket.
	symbolBucket := bucket convertToSymbols.
	typesBucket := symbolBucket convertToTypes.
	self assert: (bucket instVarNamed: 'methodTypeMaps') = 
				(typesBucket instVarNamed: 'methodTypeMaps').
	self assert: (bucket instVarNamed: 'methodTempMaps') size = 
				(typesBucket instVarNamed: 'methodTempMaps') size.
	self assert: (bucket instVarNamed: 'methodReturnMaps') = 
				(typesBucket instVarNamed: 'methodReturnMaps').
	self assert: (bucket instVarNamed: 'memberTypeMaps') = 
				(typesBucket instVarNamed: 'memberTypeMaps').
	self assert: (bucket instVarNamed: 'stackpMaps') = 
				(typesBucket instVarNamed: 'stackpMaps').! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:49.846'!
testDictionary
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testDictionary].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: Dictionary 
						elements: SmallInteger 
						keys: self byteSymbol)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:53.686'!
testDictionaryChange
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testDictionaryChange].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: Dictionary 
						elements: Number 
						keys: self byteSymbol)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:00:57.445'!
testIdentitySet
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testIdentitySet].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: IdentitySet 
						elements: Character 
						keys: nil)! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:01:02.615'!
testLocalTypes
	| m |
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testLocals].	
	m := THSBClassA >> #locals:which:.
	self
		assert: (self harvester bucket parameterTypesFor: m) size > 0;
		assert: (self harvester bucket parameterTypeAt: 1 in: m) = Object;
		assert: (self harvester bucket parameterTypeAt: 2 in: m) = SmallInteger;
		assert: (self harvester bucket localTypeAt: 1 in: m) = Number;
		assert: (self harvester bucket localTypeAt: 2 in: m) = self string;
		assert: (self harvester bucket localTypeAt: 3 in: m) = SmalltalkImage;
		assert: (self harvester bucket localTypeAt: 4 in: m) = Object! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:01:06.646'!
testMemberTypes
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testMembers].
	self
		assert: (self harvester bucket memberTypeFor: 1 in: THSBClassA) = Number;
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = self string;
		assert: (self harvester bucket memberTypeFor: 3 in: THSBClassA) = SmalltalkImage! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/3/2011 13:25:50.53'!
testMemberTypesSubclass
	self harvester harvest: [ THSBClassSubB new ].
	self
		assert: (self harvester bucket memberTypeFor: 1 in: THSBClassB) = 
					self byteString! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:01:16.997'!
testMoreParameterTypes
	| m |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testStringsMagnitudesAndObjects].
	m := THSBClassA >> #strings:magnitudes:andObjects:index:.
	self
		assert: (self harvester bucket parameterTypesFor: m) size > 0;
		assert: (self harvester bucket parameterTypeAt: 1 in: m) = self string;
		assert: (self harvester bucket parameterTypeAt: 2 in: m) = Magnitude;
		assert: (self harvester bucket parameterTypeAt: 3 in: m) = Object! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:01:21.27'!
testOrderedCollectionWithNumber
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testOrderedCollectionWithNumber].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: OrderedCollection 
						elements: Number 
						keys: nil)! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:01:26.758'!
testOrderedCollectionWithSmallInteger
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testOrderedCollectionWithSmallInteger].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: OrderedCollection 
						elements: SmallInteger 
						keys: nil)! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:01:30.852'!
testParameterTypes
	| m |
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testNumbers].
	m := THSBClassA >> #numbers:.
	self
		assert: (self harvester bucket parameterTypesFor: m) size > 0;
		assert: (self harvester bucket parameterTypeAt: 1 in: m) = Number! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:01:34.309'!
testQuickReturnAccessor

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testGetAccessor].
	self checkTypes: self expTypesTestGetAccessor! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:01:38.565'!
testRecursiveCollections
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testRecursiveCollections].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					(self 
						container: OrderedCollection 
						elements: ((self 
										container: Collection 
										elements: Object 
										keys: nil)) 
						keys: nil)! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/27/2010 17:53'!
testReturnTypeLeastCommon

	self harvester harvest: [
		THSBClassA new 
			numbers: 10;
			numbers: -3.5].
	self 
		assert: (self harvester bucket
					returnTypeFor: THSBClassA >> #numbers:) = Number				! !

!THMonoBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:01:46.581'!
testReturnTypes

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testReturn].
	self 
		assert: (self harvester bucket
					returnTypeFor: THSBClassA >> #numbers:) = SmallInteger;
		assert: (self harvester bucket
					returnTypeFor: THSBClassA >> #initialize) =THSBClassA;
		assert: (self harvester bucket
					returnTypeFor: THSBClassA >> #a:) = THSBClassA.! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:01:50.132'!
testString
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testString].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = 
					self byteString.! !

!THMonoBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:01:53.283'!
testStringAndSymbol
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testStringAndSymbol].
	self
		assert: (self harvester bucket memberTypeFor: 2 in: THSBClassA) = self string.! !

THTest subclass: #THPartialByteCodeOrderTest
	instanceVariableNames: 'scannedByteCodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester'!

!THPartialByteCodeOrderTest methodsFor: 'checking' stamp: 'mp 3/12/2010 16:14'!
checkByteCodes: scannedByteCodes with: expectedByteCodes

	self assert: scannedByteCodes size = expectedByteCodes size.
	scannedByteCodes withIndexDo: [:each :index |
		self assert: each first == (expectedByteCodes at: index) first. "Receiver Class"
		self assert: each second == (expectedByteCodes at: index) second. "Method Selector"
		self assert: each third == (expectedByteCodes at: index) third. "PC"
		].! !

!THPartialByteCodeOrderTest methodsFor: 'expected byte codes' stamp: 'mp 5/26/2010 16:28'!
expBCTestGetAccessor

^ {
	{THSBSandboxTest. #testGetAccessor. 25}.
	{THSBSandboxTest. #testGetAccessor. 26}.
	{THSBSandboxTest. #testGetAccessor. 27}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.
	{THSBSandboxTest. #testGetAccessor. 28}.
		"Quick return method: No ByteCode is executed!!."
	{THSBSandboxTest. #testGetAccessor. 29}.
	{THSBSandboxTest. #testGetAccessor. 30}.
	{THSBSandboxTest. #testGetAccessor. 31}.
	{THSBSandboxTest. #testGetAccessor. 32}.
	{THSBSandboxTest. #testGetAccessor. 33}.
	}! !

!THPartialByteCodeOrderTest methodsFor: 'expected byte codes' stamp: 'mp 5/26/2010 17:24'!
expBCTestLoopBlock

^ {
	{THSBSandboxTest. #testLoopBlock. 33}.
	{THSBSandboxTest. #testLoopBlock. 35}.
	{THSBSandboxTest. #testLoopBlock. 36}.
	{THSBSandboxTest. #testLoopBlock. 37}.
	{THSBSandboxTest. #testLoopBlock. 40}.
	{THSBSandboxTest. #testLoopBlock. 41}.
	{THSBSandboxTest. #testLoopBlock. 42}.
	{THSBSandboxTest. #testLoopBlock. 56}.
		"1. Loop"
		"Block is executed outside of our partial trace but we expect something like this"
	{THSBSandboxTest. #testLoopBlock. 46}.
	{THSBSandboxTest. #testLoopBlock. 47}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.	
		{THSBSandboxTest. #testLoopBlock. 48}.
		{THSBSandboxTest. #testLoopBlock. 51}.
			{THSBClassA. #addValueToA:. 13}.
			{THSBClassA. #addValueToA:. 14}.
			{THSBClassA. #addValueToA:. 15}.
			{THSBClassA. #addValueToA:. 16}.
		{THSBSandboxTest. #testLoopBlock. 52}.
		{THSBSandboxTest. #testLoopBlock. 55}.
		"2. Loop"
		"Block is executed outside of our partial trace but we expect something like this"
	{THSBSandboxTest. #testLoopBlock. 46}.
	{THSBSandboxTest. #testLoopBlock. 47}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.	
		{THSBSandboxTest. #testLoopBlock. 48}.
		{THSBSandboxTest. #testLoopBlock. 51}.
			{THSBClassA. #addValueToA:. 13}.
			{THSBClassA. #addValueToA:. 14}.
			{THSBClassA. #addValueToA:. 15}.
			{THSBClassA. #addValueToA:. 16}.
		{THSBSandboxTest. #testLoopBlock. 52}.
		{THSBSandboxTest. #testLoopBlock. 55}.
	{THSBSandboxTest. #testLoopBlock. 57}.
	{THSBSandboxTest. #testLoopBlock. 58}.
	{THSBSandboxTest. #testLoopBlock. 59}.	
	{THSBSandboxTest. #testLoopBlock. 62}.
	{THSBSandboxTest. #testLoopBlock. 63}.
	{THSBSandboxTest. #testLoopBlock. 64}.
	{THSBSandboxTest. #testLoopBlock. 65}.
	{THSBSandboxTest. #testLoopBlock. 66}.
	}! !

!THPartialByteCodeOrderTest methodsFor: 'expected byte codes' stamp: 'mp 5/26/2010 17:08'!
expBCTestReturn

	^ {
	{THSBSandboxTest. #testReturn. 25}.
	{THSBSandboxTest. #testReturn. 26}.
	{THSBSandboxTest. #testReturn. 27}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.	
	{THSBSandboxTest. #testReturn. 28}.
	{THSBSandboxTest. #testReturn. 29}.
		{THSBClassA. #numbers:. 13}.
		{THSBClassA. #numbers:. 14}.
	{THSBSandboxTest. #testReturn. 30}.
	{THSBSandboxTest. #testReturn. 31}.
	{THSBSandboxTest. #testReturn. 32}.
	{THSBSandboxTest. #testReturn. 33}.
	{THSBSandboxTest. #testReturn. 34}.
	}! !

!THPartialByteCodeOrderTest methodsFor: 'expected byte codes' stamp: 'mp 5/26/2010 18:01'!
expBCTestTraceMethodExecution

	^ {
	{THSBSandboxTest. #testTraceMethodExecution. 33}.
	{THSBSandboxTest. #testTraceMethodExecution. 34}.
	{THSBSandboxTest. #testTraceMethodExecution. 35}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.	
	{THSBSandboxTest. #testTraceMethodExecution. 36}.
	{THSBSandboxTest. #testTraceMethodExecution. 37}.	
		{THSBClassA. #sayHelloTo:. 37}.
		{THSBClassA. #sayHelloTo:. 38}.		
			{THSBClassB. #initialize. 21}.
			{THSBClassB. #initialize. 22}.
			{THSBClassB. #initialize. 23}.
				{THSBClassB. #mark:. 13}.
				{THSBClassB. #mark:. 14}.
				{THSBClassB. #mark:. 15}.	
			{THSBClassB. #initialize. 24}.	
			{THSBClassB. #initialize. 25}.
		{THSBClassA. #sayHelloTo:. 39}.
		{THSBClassA. #sayHelloTo:. 40}.
		{THSBClassA. #sayHelloTo:. 41}.
		{THSBClassA. #sayHelloTo:. 42}.
			{THSBClassB. #getHello. 17}.
			{THSBClassB. #getHello. 18}.
		{THSBClassA. #sayHelloTo:. 43}.
		{THSBClassA. #sayHelloTo:. 44}.
		{THSBClassA. #sayHelloTo:. 45}.
		{THSBClassA. #sayHelloTo:. 46}.
		{THSBClassA. #sayHelloTo:. 47}.
		{THSBClassA. #sayHelloTo:. 48}.		
			{THSBClassB. #addExclamationMarkTo:. 17}.
			{THSBClassB. #addExclamationMarkTo:. 18}.
			{THSBClassB. #addExclamationMarkTo:. 19}.						
			{THSBClassB. #addExclamationMarkTo:. 20}.
		{THSBClassA. #sayHelloTo:. 49}.
	{THSBSandboxTest. #testTraceMethodExecution. 38}.
	{THSBSandboxTest. #testTraceMethodExecution. 39}.
	{THSBSandboxTest. #testTraceMethodExecution. 40}.
	{THSBSandboxTest. #testTraceMethodExecution. 41}.
	{THSBSandboxTest. #testTraceMethodExecution. 42}.			
	}! !

!THPartialByteCodeOrderTest methodsFor: 'expected byte codes' stamp: 'mp 5/26/2010 18:13'!
expBCTestTraceMethodExecutionBlock

	^ {
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 41}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 42}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 43}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.		
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 44}.
	"Block Byte Code - Not executed at this point"
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 56}.
		{THSBClassA. #concatTwoStrings:. 25}.
		{THSBClassA. #concatTwoStrings:. 26}.
		{THSBClassA. #concatTwoStrings:. 27}.
		{THSBClassA. #concatTwoStrings:. 28}.
			"Block is executed and we expect something like this"
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 48}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 49}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 50}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 51}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 52}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 53}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 54}.
			{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 55}.
		{THSBClassA. #concatTwoStrings:. 29}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 57}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 58}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 59}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 60}.
	{THSBSandboxTest. #testTraceMethodExecutionWithBlock. 61}.		
	}! !

!THPartialByteCodeOrderTest methodsFor: 'expected byte codes' stamp: 'mp 5/26/2010 18:18'!
expBCTestTraceMethodExecutionConditional

	^ {
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 45}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 46}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 47}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 48}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 49}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 50}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 51}.
		{THSBClassA. #initialize. 17}.
		{THSBClassA. #initialize. 18}.
		{THSBClassA. #initialize. 19}.
			{THSBClassA. #a:. 13}.
			{THSBClassA. #a:. 14}.
			{THSBClassA. #a:. 15}.
		{THSBClassA. #initialize. 20}.
		{THSBClassA. #initialize. 21}.		
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 52}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 53}.	
		{THSBClassA. #sayHelloTo:. 37}.
		{THSBClassA. #sayHelloTo:. 38}.		
			{THSBClassB. #initialize. 21}.
			{THSBClassB. #initialize. 22}.
			{THSBClassB. #initialize. 23}.
				{THSBClassB. #mark:. 13}.
				{THSBClassB. #mark:. 14}.
				{THSBClassB. #mark:. 15}.	
			{THSBClassB. #initialize. 24}.	
			{THSBClassB. #initialize. 25}.
		{THSBClassA. #sayHelloTo:. 39}.
		{THSBClassA. #sayHelloTo:. 40}.
		{THSBClassA. #sayHelloTo:. 41}.
		{THSBClassA. #sayHelloTo:. 42}.
			{THSBClassB. #getHello. 17}.
			{THSBClassB. #getHello. 18}.		
		{THSBClassA. #sayHelloTo:. 43}.
		{THSBClassA. #sayHelloTo:. 44}.
		{THSBClassA. #sayHelloTo:. 45}.
		{THSBClassA. #sayHelloTo:. 46}.
		{THSBClassA. #sayHelloTo:. 47}.
		{THSBClassA. #sayHelloTo:. 48}.		
			{THSBClassB. #addExclamationMarkTo:. 17}.
			{THSBClassB. #addExclamationMarkTo:. 18}.
			{THSBClassB. #addExclamationMarkTo:. 19}.						
			{THSBClassB. #addExclamationMarkTo:. 20}.
		{THSBClassA. #sayHelloTo:. 49}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 54}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 56}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 60}.		
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 61}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 62}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 63}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 64}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 65}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 66}.
	{THSBSandboxTest. #testTraceMethodExecutionWithCondition. 67}.						
	}! !

!THPartialByteCodeOrderTest methodsFor: 'running' stamp: 'mp 5/26/2010 10:59'!
setUp
	super setUp.
	scannedByteCodes := OrderedCollection new.
	self harvester 
		eager;
		harvesterBlock: [:ctx | 
			scannedByteCodes add: {ctx receiver class. ctx method selector. ctx pc}].
	! !

!THPartialByteCodeOrderTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:06:43.69'!
testGetAccessor

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testGetAccessor].
	self checkByteCodes: scannedByteCodes with: self expBCTestGetAccessor.! !

!THPartialByteCodeOrderTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:06:48.41'!
testLoopBlock

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testLoopBlock].
	self checkByteCodes: scannedByteCodes with: self expBCTestLoopBlock.! !

!THPartialByteCodeOrderTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:06:52.682'!
testReturn

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testReturn].
	self checkByteCodes: scannedByteCodes with: self expBCTestReturn.! !

!THPartialByteCodeOrderTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:07:01.657'!
testTraceMethodExecution

	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testTraceMethodExecution].
	self checkByteCodes: scannedByteCodes 
		 with: self expBCTestTraceMethodExecution.
! !

!THPartialByteCodeOrderTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:07:05.577'!
testTraceMethodExecutionWithBlock

	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testTraceMethodExecutionWithBlock].
	self checkByteCodes: scannedByteCodes 
		 with: self expBCTestTraceMethodExecutionBlock.! !

!THPartialByteCodeOrderTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:07:09.034'!
testTraceMethodExecutionWithCondition

	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testTraceMethodExecutionWithCondition].
	self checkByteCodes: scannedByteCodes 
		 with: self expBCTestTraceMethodExecutionConditional.! !

THTest subclass: #THPolyBucketTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester'!

!THPolyBucketTest methodsFor: 'running' stamp: 'mp 5/5/2011 17:10:27.715'!
expectedFailures
	^ #( #testReturnTypeQuick 
		#testHarvestDictionaryChange 
		#testHarvestIncrementalAddOfNumbers)! !

!THPolyBucketTest methodsFor: 'running' stamp: 'mp 5/26/2010 11:03'!
setUp
	super setUp.
	self harvester eager.! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:07:27.482'!
testClosureTemps

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testCollectCollection].
	self 
		assert: (self harvester bucket 
					tempDegreeFor: THSBClassA >> #collectCollection: at: 1) = 1;
		assert: (self harvester bucket 
					tempSetFor: THSBClassA >> #collectCollection: at: 1) = 
						(Set with: (self container: Array elements: {SmallInteger} keys: {}))! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:11:32.278'!
testContainerArguments
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 1) anyOne.
	self assert: type = (self container: Association 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 2) anyOne.	
	self assert: type = (self container: OrderedCollection 
									elements: {SmallInteger. self byteString} 
									keys: {}).
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 3) anyOne.
	self assert: type = (self container: Dictionary 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 4) anyOne.
	self assert: type = SmallInteger.	! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:07:40.505'!
testContainerInstVars
	| types container |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	types := (self harvester bucket memberTypeSetFor: THSBClassA at: 2).
	self assert: types size = 4.
	self assert: (types includes: SmallInteger).
	container := types detect: [:each | each isBehavior
									ifTrue: [false]
									ifFalse: [each typeOfContainer = OrderedCollection]].
	self assert: container = (self container: OrderedCollection 
									elements: {SmallInteger. self byteString} 
									keys: {}).
	container := types detect: [:each | each isBehavior
										ifTrue: [false]
										ifFalse: [each typeOfContainer = Dictionary]].
	self assert: container = (self container: Dictionary 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).
	container := types detect: [:each | each isBehavior
										ifTrue: [false]
										ifFalse: [each typeOfContainer = Association]].
	self assert: container = (self container: Association 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).			! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:08:44.637'!
testContainerReturn
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	type := (self harvester bucket 
					returnTypeSetFor: THSBClassA>>#assoc:collection:dict:number:) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Array;
		assert: type typeOfElements = (Set 
											with: (self container: OrderedCollection 
													     elements: {SmallInteger. 
																    self byteString} 
													     keys: {})
											with: SmallInteger
											with: (self container: Dictionary 
													     elements: {SmallInteger} 
													     keys: {self byteSymbol})
											with: (self container: Association 
													     elements: {SmallInteger} 
													     keys: {self byteSymbol}));
		assert: type typeOfKeys isEmpty.									! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:09:39.38'!
testContainerTemps
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 5) anyOne.
	self assert: type = (self container: Association 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 6) anyOne.	
	self assert: type = (self container: OrderedCollection 
									elements: {SmallInteger. self byteString} 
									keys: {}).
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 7) anyOne.
	self assert: type = (self container: Dictionary 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).
	type := (self harvester bucket 
				tempSetFor: THSBClassA>>#assoc:collection:dict:number: at: 8) anyOne.
	self assert: type = SmallInteger.																										! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:09:43.059'!
testContainerTopOfStack
	| initialPC type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMethodWithContainersAllOver].
	initialPC := (THSBClassA>>#assoc:collection:dict:number:) initialPC.
	type := (self harvester bucket 
				typesAt: initialPC + 4 - 1 
				in: THSBClassA>>#assoc:collection:dict:number:) anyOne.
	self assert: type = (self container: OrderedCollection 
									elements: {SmallInteger. self byteString} 
									keys: {}).				
	type := (self harvester bucket 
				typesAt: initialPC + 6 - 1 
				in: THSBClassA>>#assoc:collection:dict:number:) anyOne.
	self assert: type = (self container: Dictionary 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).
	type := (self harvester bucket 
				typesAt: initialPC + 9 - 1 
				in: THSBClassA>>#assoc:collection:dict:number:) anyOne.
	self assert: type = SmallInteger.
	type := (self harvester bucket 
				typesAt: initialPC + 11 - 1 
				in: THSBClassA>>#assoc:collection:dict:number:) anyOne.
	self assert: type = (self container: Association 
									elements: {SmallInteger} 
									keys: {self byteSymbol}).! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:09:47.329'!
testHarvestArrayWithObject
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testArrayWithObject].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Array;
		assert: type typeOfElements  = (Set 
											with: self byteString 
											with: SmallInteger);
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:09:54.781'!
testHarvestAssociation
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testAssociation].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Association;
		assert: type typeOfElements anyOne = self byteString;
		assert: type typeOfKeys anyOne = self byteSymbol.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:09:58.377'!
testHarvestAssociationChange
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testAssociationChange].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Association;
		assert: type typeOfElements = (Set 
											with: self byteString 
											with: SmallInteger);
		assert: type typeOfKeys = (Set 
										with: self byteSymbol 
										with: SmallInteger).! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:05.429'!
testHarvestAssociationChangeElement
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testAssociationChangeElement].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Association;
		assert: type typeOfElements = (Set 
										with: self byteString 
										with: SmallInteger);
		assert: type typeOfKeys = (Set with: self byteSymbol).! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:16.892'!
testHarvestAssociationChangeKey
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testAssociationChangeKey].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Association;
		assert: type typeOfElements = (Set with: self byteString);
		assert: type typeOfKeys = (Set 
										with: self byteSymbol 
										with: SmallInteger).! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:24.003'!
testHarvestBagWithNumbers
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testBagWithNumbers].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Bag;
		assert: type typeOfElements = (Set with: SmallInteger with: Float);
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:08:29.063'!
testHarvestChangeFromArrayToOrderedCollection
	| types container |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testChangeFromArrayToOrderedCollection].
	types := (self harvester bucket memberTypeSetFor: THSBClassA at: 2).
	self assert: types size = 2.
	container := types detect: [:each | each typeOfContainer = Array].
	self assert: container = (self container: Array
											   elements: {SmallInteger} 
											   keys: {}).
	container := types detect: [:each | each typeOfContainer = OrderedCollection].
	self assert: container = (self container: OrderedCollection
											   elements: {SmallInteger} 
											   keys: {}).												! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'mp 5/5/2011 17:08:33.272'!
testHarvestChangeFromSingleObjectToCollection
	| types container |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testChangeFromSingleObjectToCollection].
	types := (self harvester bucket memberTypeSetFor: THSBClassA at: 2).
	self assert: types size = 2.
	self assert: (types includes: SmallInteger).
	container := types detect: [:each | each isBehavior not].
	self assert: container = (self container: OrderedCollection
											   elements: {SmallInteger} 
											   keys: {}).! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:28.913'!
testHarvestDictionary
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testDictionary].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Dictionary;
		assert: type typeOfElements anyOne = SmallInteger;
		assert: type typeOfKeys anyOne = self byteSymbol.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:32.469'!
testHarvestDictionaryChange
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testDictionaryChange].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Dictionary;
		assert: type typeOfElements = (Set with: SmallInteger with: Float);
		assert: type typeOfKeys = (Set with: self byteSymbol).! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:36.71'!
testHarvestIdentitySet
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testIdentitySet].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = IdentitySet;
		assert: type typeOfElements anyOne = Character;
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:46.915'!
testHarvestIncrementalAddOfNumbers
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testIncrementalAddOfNumbers].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = OrderedCollection;
		assert: type typeOfElements = (Set with: SmallInteger with: Float with: Fraction);
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:50.574'!
testHarvestInterval
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testInterval].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Interval;
		assert: type typeOfElements anyOne = SmallInteger;
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:53.992'!
testHarvestMatrix
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testMatrix].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Matrix;
		assert: type typeOfElements anyOne = SmallInteger;
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:10:58.164'!
testHarvestOrderedCollectionWithNumber
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testOrderedCollectionWithNumber].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = OrderedCollection;
		assert: type typeOfElements = (Set with: SmallInteger with: Float);
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:11:01.691'!
testHarvestOrderedCollectionWithSmallInteger
	| type |
	self harvester harvest: [THSBSandboxTest 
							runWithoutTimeout: #testOrderedCollectionWithSmallInteger].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = OrderedCollection;
		assert: type typeOfElements anyOne = SmallInteger;
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:11:05.057'!
testHarvestRecursiveCollections
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testRecursiveCollections].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = OrderedCollection;
		assert: type typeOfElements = (Set 
								with: (self container: Array 
										    elements: {SmallInteger}
										    keys: {}) 
								with: (self container: Array 
										    elements: {Character} 
										    keys: {})
								with: (self container: Set 
										    elements: {self byteString} 
										    keys: {}));
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:11:09.007'!
testHarvestSetWithIntegers
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testSetWithIntegers].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = Set;
		assert: type typeOfElements anyOne = SmallInteger;
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing-container' stamp: 'tfel 5/9/2011 10:11:12.881'!
testHarvestString
	| type |
	self harvester harvest: [THSBSandboxTest 
								runWithoutTimeout: #testString].
	type := (self harvester bucket memberTypeSetFor: THSBClassA at: 2) anyOne.
	self
		deny: type isBehavior;
		assert: type class = THContainerType;
		assert: type typeOfContainer = ByteString;
		assert: type typeOfElements anyOne = Character;
		assert: type typeOfKeys isEmpty.! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:09:17.91'!
testLocalTypes
	| m |
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testLocals].
	m := THSBClassA >> #locals:which:.
	self
		assert: (self harvester bucket localDegreeAt: 1 in: m) = 2;
		assert: (self harvester bucket localDegreeAt: 2 in: m) = 2;
		assert: (self harvester bucket localDegreeAt: 3 in: m) = 1;
		assert: (self harvester bucket localDegreeAt: 4 in: m) = 5! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:09:21.334'!
testMemberTypes
	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testMembers].
	self
		assert: (self harvester bucket 
					memberTypeDegreeFor: THSBClassA at: 1) = 2;
		assert: (self harvester bucket 
					memberTypeDegreeFor: THSBClassA at: 2) = 2;
		assert: (self harvester bucket 
					memberTypeDegreeFor: THSBClassA at: 3) = 1! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/27/2010 16:54'!
testReturnTypeBlock

	self harvester harvest: [THSBClassA new returnBlock: 1].
	self 
		assert: (self harvester bucket
					returnDegreeFor: THSBClassA >> #returnBlock:) = 1;
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #returnBlock:) = 
						(Set with: SmallInteger);
		assert: (self harvester bucket
					returnDegreeFor: THSBClassA >> #evalBlock:with:) = 0						! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/27/2010 17:39'!
testReturnTypeQuick

	self harvester harvest: [THSBClassA new a].
	self 
		assert: (self harvester bucket
					returnDegreeFor: THSBClassA >> #a) = 1;
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #a) = (Set with: SmallInteger)				! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/3/2011 11:10:21.066'!
testReturnTypeSets

	self harvester harvest: [
		THSBClassA new 
			numbers: 10;
			numbers: -3.5;
			numbers: 'abc';
			numbers: THSBClassB new].
	self 
		assert: (self harvester bucket
					returnDegreeFor: THSBClassA >> #numbers:) = 4;
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #numbers:) = 
						(Set 
							with: SmallInteger
							with: Float
							with: self byteString
							with: THSBClassB)				! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/27/2010 16:44'!
testReturnTypeSpecialConstants

	self harvester harvest: [THSBClassA new returnTrue].
	self 
		assert: (self harvester bucket
					returnDegreeFor: THSBClassA >> #returnTrue) = 1;
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #returnTrue) = (Set with: True)				! !

!THPolyBucketTest methodsFor: 'testing' stamp: 'mp 5/5/2011 17:09:31.414'!
testReturnTypes

	self harvester harvest: [THSBSandboxTest runWithoutTimeout: #testReturn].
	self 
		assert: (self harvester bucket
					returnDegreeFor: THSBClassA >> #numbers:) = 1;
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #numbers:) = (Set with: SmallInteger);
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #initialize) = (Set with: THSBClassA);
		assert: (self harvester bucket
					returnTypeSetFor: THSBClassA >> #a:) = (Set with: THSBClassA).					! !

!THTest class methodsFor: 'constants' stamp: 'tfel 3/6/2012 15:22:58.801'!
selectedCategories

	^ #(#'Path-Tests-Harvester-Sandbox')! !

!THTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 11:07:53.416'!
byteString
	^ self container: ByteString
		elements: {Character} 
		keys: {}! !

!THTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 11:07:46.499'!
byteSymbol
	^ self container: ByteSymbol 
		elements: {Character} 
		keys: {}! !

!THTest methodsFor: 'accessing' stamp: 'mp 5/3/2011 14:43:36.45'!
container: aClass elements: elementTypes keys: keyTypes
	| container |
	container := THContainerType forPoly: aClass.
	elementTypes do: [:each | container typeOfElements add: each].
	keyTypes do: [:each | container typeOfKeys add: each].
	^ container! !

!THTest methodsFor: 'accessing' stamp: 'mp 5/26/2010 10:42'!
harvester
	"Answer the value of harvester"

	^ harvester! !

!THTest methodsFor: 'accessing' stamp: 'mp 5/26/2010 10:42'!
harvester: anObject
	"Set the value of harvester"

	harvester := anObject! !

!THTest methodsFor: 'running' stamp: 'mp 1/24/2012 16:20:54.379'!
setUp

	self harvester: (THTypeHarvester forCategories: self class selectedCategories).! !

Spec subclass: #SPVMorphTest
	instanceVariableNames: 'klass testClass instance previousProject testProject associatedTestClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

SPVMorphTest subclass: #SClassDiagramTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SClassDiagramTest methodsFor: 'transfer-event-helpers' stamp: 'tfel 4/28/2011 16:40:32.028'!
defaultTransferType

	^ #dragTransferPlus! !

!SClassDiagramTest methodsFor: 'drop-event-helpers' stamp: 'tfel 4/28/2011 16:46:39.646'!
dropEventFor: aMorph

	^ DropEvent new
		setPosition: instance position + (1 @ 1)
		contents: aMorph
		hand: ActiveHand;
		yourself! !

!SClassDiagramTest methodsFor: 'transfer-event-helpers' stamp: 'tfel 4/28/2011 16:40:41.757'!
omniBrowserTransferType

	^ #OmniBrowser! !

!SClassDiagramTest methodsFor: 'running' stamp: 'tfel 4/28/2011 16:01:39.696'!
setUp

	super setUp.
	instance := klass new
					openInWorld;
					bottomRight: 0@0; "Do not show the Morph. Making it really hidden
										will prevent things like DropEvents"
					yourself.
! !

!SClassDiagramTest methodsFor: 'testing' stamp: 'tfel 7/14/2011 15:03:04.021'!
testAcceptsClassDropsFromBrowser

	self mock: #confirm: of: UIManager default class toReturn: false during: [
		self assert: [ instance submorphs isEmpty ].
		(self transferEventFor: self class) sentTo: instance.
		self assert: [ instance submorphs size = 1 ].
		self assert: [ instance submorphs first class = SClassMorph ].
		self assert: [ instance submorphs first klass = self class ]].
! !

!SClassDiagramTest methodsFor: 'testing' stamp: 'tfel 7/14/2011 15:03:57.303'!
testAcceptsClassDropsFromOmniBrowser
	| OBClassNodeClass |
	OBClassNodeClass := Smalltalk at: #OBClassNode ifAbsent: [].
	OBClassNodeClass ifNil: [^ self assert: true].
	self mock: #confirm: of: UIManager default class toReturn: false during: [
		self assert: [instance submorphs isEmpty].
		(self 
			transferEventFor: (OBClassNodeClass on: self class) 
			withType: self omniBrowserTransferType) sentTo: instance.
		self assert: [instance submorphs size = 1].
		self assert: [instance submorphs first class = SClassMorph].
		self assert: [instance submorphs first klass = self class]].
! !

!SClassDiagramTest methodsFor: 'testing' stamp: 'tfel 7/14/2011 15:03:19.449'!
testFailsSilentlyForRandomlyDroppedMorphs

	self mock: #confirm: of: UIManager default class toReturn: false during: [
		self assert: [instance submorphs isEmpty].
		(self transferEventFor: (TransferMorph
									withPassenger: self class methodDict values first)) 
			sentTo: instance.
		self assert: [instance submorphs isEmpty].
		(self dropEventFor: (BorderedMorph new)) sentTo: instance.
		self assert: [instance submorphs isEmpty]].
! !

!SClassDiagramTest methodsFor: 'testing' stamp: 'tfel 7/14/2011 15:03:42.754'!
testTransferedMorphHasDropPosition
	| dropEvent dropPosition |
	self mock: #confirm: of: UIManager default class toReturn: false during: [
		self assert: [ instance submorphs isEmpty ].
		dropPosition := 27@36.
		dropEvent := self transferEventFor: self class.
		dropEvent contents position: dropPosition.
		dropEvent sentTo: instance.
		self assert: [ instance submorphs size = 1 ].
		self assert: [ instance submorphs first class = SClassMorph ].
		self assert: [ instance submorphs first klass = self class ].
		self assert: [ instance submorphs first position = dropPosition]].! !

!SClassDiagramTest methodsFor: 'transfer-event-helpers' stamp: 'tfel 4/28/2011 16:47:10.507'!
transferEventFor: aPassenger

	^ self transferEventFor: aPassenger withType: self defaultTransferType! !

!SClassDiagramTest methodsFor: 'transfer-event-helpers' stamp: 'tfel 4/28/2011 16:47:16.837'!
transferEventFor: aPassenger withType: aTransferType

	^ self dropEventFor: ((TransferMorph withPassenger: aPassenger)
							dragTransferType: aTransferType;
							yourself)! !

SPVMorphTest subclass: #SClassMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SClassMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/17/2011 15:46:25.872'!
expectedFailures

	^ #(testShouldAllowChangingTheSuperclass)! !

!SClassMorphTest methodsFor: 'running' stamp: 'tfel 4/30/2011 14:16:59.558'!
setUp

	super setUp.
	instance := klass on: testClass.! !

!SClassMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 7/14/2011 15:09:38.001'!
testOpenCloseAncestors
	"Test auto-opening of ancestors. (instance + ((connector + superclass) * auto ancestor depth))"
	| pasteUp arrowMorphs classMorphs superclasses oldPreference |
	oldPreference := SClassMorph autoAncestorDepth.
	SClassMorph autoAncestorDepth: 2.
	pasteUp := SClassDiagram new.
	pasteUp addMorph: instance.
	self assert: [pasteUp submorphs size = 1].
	instance openAncestors.
	classMorphs := pasteUp submorphs select: [:m | m isKindOf: SClassMorph].
	arrowMorphs := pasteUp submorphs select: [:m | m isKindOf: SInheritanceConnector].
	self assert: [classMorphs size = (1 + SClassMorph autoAncestorDepth)].
	self assert: [arrowMorphs size = SClassMorph autoAncestorDepth].
	superclasses := (instance klass withAllSuperclasses copyFrom: 1 to: SClassMorph autoAncestorDepth + 1).
	classMorphs do: [:classMorph |
		self assert: [superclasses includes: classMorph klass].
		superclasses remove: classMorph klass].
	SClassMorph autoAncestorDepth: oldPreference.
! !

!SClassMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 2/14/2012 10:54:59.826'!
testOpenDirectAssociations
	"Test auto-opening of associations"
	| pasteUp arrowMorphs classMorphs submorphs classMorphClasses |
	pasteUp := SClassDiagram new.
	pasteUp addMorph: instance.
	self assert: [pasteUp submorphs size = 1].
	instance openAssociations.
	classMorphs := pasteUp submorphs select: [:m | m isKindOf: SClassMorph].
	arrowMorphs := pasteUp submorphs select: [:m | m isKindOf: SAssociationConnector].
	"The testClass has 2 instance variables, ivar1 and ivar2"
	self assert: [classMorphs size = 3].
	self assert: [arrowMorphs size = 2].
	classMorphClasses := classMorphs collect: [:classMorph | classMorph klass].
	{testClass . associatedTestClass . associatedTestClass superclass} do: [:class |
		self assert: (classMorphClasses includes: class)].
	((self allSubmorphsRecursiveFor: instance)
		detect: [:m | m isKindOf: SStateMorph]) collapseMorphs.
	submorphs := (self allSubmorphsRecursiveFor: pasteUp).
	self assert: [ self thereIsAStringMorphIn: submorphs with: 'ivar1'].
	self assert: [ self thereIsAStringMorphIn: submorphs with: 'ivar2'].
! !

!SClassMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/13/2011 15:49:42.945'!
testShouldAllowChangingTheSuperclass
	| newSuperclassMorph oldSuperclass |
	newSuperclassMorph := klass on: Morph.
	oldSuperclass := testClass superclass.
	instance newSuperclassTarget: newSuperclassMorph.
	[self assert: [testClass superclass = Morph]]
		ensure: [klass superclass: oldSuperclass].! !

!SClassMorphTest methodsFor: 'as yet unclassified' stamp: 'ag 9/22/2011 14:19:04.636'!
testShouldReturnItselfAsClassMorph

	self assert: [instance classMorph = instance].! !

SPVMorphTest subclass: #SEntryMorphTest
	instanceVariableNames: 'classMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

SEntryMorphTest subclass: #SClassNameMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SClassNameMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 6/9/2011 14:05:12.048'!
testShouldAllowOpeningABrowserOnDoubleClick
	<ignoreForTypeHarvesting>
	self mock: #browse of: Class toReturn: [:v | v] during: [
		self
			assert: [instance doubleClick: MorphicUnknownEvent new]
			sends: #browse
			to: Class].! !

!SClassNameMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 18:27:04.694'!
testShouldAllowRefreshingTheClass
	| set newName |
	set := self allSubmorphsRecursiveFor: instance.
	newName := self class testClassName, 'Renamed'.
	instance update.
	self assert: [ self thereIsAStringMorphIn: set with: self class testClassName ].
	testClass rename: newName.
	instance update.
	[self assert: [ self thereIsAStringMorphIn: set with: newName ]]
		ensure: [testClass rename: self class testClassName].! !

!SClassNameMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 18:27:39.005'!
testShouldAutomaticallyRefreshTheClass
	| set newName |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	newName := self class testClassName, 'Renamed'.
	self assert: [ self thereIsAStringMorphIn: set with: self class testClassName ].
	testClass rename: newName.
	[self assert: [ self thereIsAStringMorphIn: set with: newName ]]
		ensure: [testClass rename: self class testClassName].! !

!SClassNameMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 18:11:03.415'!
testShouldBeExpandedByDefault
	
	instance update.
	self assert: [instance entries allSatisfy: [:morph | morph owner = instance]].! !

!SClassNameMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 18:29:29.951'!
testShouldShowAClassName
	| set |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [ self thereIsAStringMorphIn: set with: self class testClassName ]! !

!SClassNameMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 18:29:20.685'!
testShouldShowASuperclass
	| set |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [ self thereIsAStringMorphIn: set with: testClass superclass name asString ]! !

!SEntryMorphTest class methodsFor: 'testing' stamp: 'tfel 5/13/2011 17:57:28.548'!
isAbstract

	^ self name = #SEntryMorphTest! !

!SEntryMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 2/14/2012 11:23:26.583'!
aboutToCreateClassMorphFor: aTestClass
	"Hook to run just before trying to creating a class morph"! !

!SEntryMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 5/5/2011 14:32:03.79'!
aboutToSearchInstanceIn: aClassMorph
	"Hook to run just before trying to detect the instance in a class morph"! !

!SEntryMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 5/5/2011 14:28:28.077'!
detectInstanceIn: aClassMorph
	"Hook for those submorphs that need to do sth special to find themselves in a SClassMorph"
	
	self subclassResponsibility.! !

!SEntryMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 5/24/2011 13:59:31.954'!
expandMorphsIn: aClassMorph

	aClassMorph allMorphsDo: [:s | (s respondsTo: #expandMorphs) ifTrue: [s expandMorphs]].! !

!SEntryMorphTest methodsFor: 'running' stamp: 'tfel 2/14/2012 11:22:26.5'!
setUp
	
	super setUp.
	self aboutToCreateClassMorphFor: testClass.
	classMorph := SClassMorph on: testClass.
	self aboutToSearchInstanceIn: classMorph.
	instance := classMorph allMorphs
		detect: [:m | m class = klass and: [m klass = testClass]]
		ifNone: [self detectInstanceIn: classMorph].! !

SEntryMorphTest subclass: #SExpandableEntryMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SExpandableEntryMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 2/14/2012 11:01:42.464'!
detectInstanceIn: aClassMorph

	^ aClassMorph allMorphs detect: [:m | m class = SStateMorph]! !

!SExpandableEntryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 2/14/2012 11:04:06.495'!
setUp

	super setUp.
	instance := SStateMorph on: testClass.
	instance type: #instance.! !

!SExpandableEntryMorphTest methodsFor: 'running' stamp: 'tfel 4/30/2011 18:08:46.752'!
tearDown

	instance abandon.
	super tearDown.! !

!SExpandableEntryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 18:12:32.598'!
testShouldAllowCollapsing

	instance update.
	instance expandMorphs.
	self assert: [instance entries allSatisfy: [:morph | morph owner = instance]].
	instance collapseMorphs.
	self assert: [instance entries allSatisfy: [:morph | morph owner isNil]].! !

!SExpandableEntryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 2/14/2012 11:04:02.45'!
testShouldAllowExpansion

	instance update.
	instance collapseMorphs.
	self assert: [instance entries allSatisfy: [:morph | morph owner isNil]].
	instance expandMorphs.
	self assert: [instance entries allSatisfy: [:morph | morph owner = instance]].! !

SEntryMorphTest subclass: #SInstVarMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SInstVarMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 5/5/2011 14:35:13.2'!
aboutToSearchInstanceIn: aClassMorph

	self expandMorphsIn: aClassMorph.! !

!SInstVarMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/17/2011 15:47:06.643'!
expectedFailures

	^ #(testShouldAnswerAnAssociationIfTheClassIsADictionary)! !

!SInstVarMorphTest methodsFor: 'get class tests' stamp: 'tfel 5/5/2011 17:29:04.134'!
testShouldAnswerAnAssociationIfTheClassIsADictionary

	(testProject typesBucket memberTypeMapFor: testClass) 
		atAllPut: (THContainerType new
					typeOfContainer: Dictionary new;
					typeOfElements: testClass;
					typeOfKeys: Symbol;
					yourself).
	[self assert: [instance getClass = (Symbol -> testClass)]]
		ensure: [(testProject typesBucket memberTypeMapFor: testClass) atAllPut: testClass]! !

!SInstVarMorphTest methodsFor: 'get class tests' stamp: 'tfel 2/14/2012 11:15:33.437'!
testShouldAnswerTheClassIfItKnowsIt

	self assert: [instance getClass = associatedTestClass].! !

!SInstVarMorphTest methodsFor: 'get class tests' stamp: 'tfel 2/14/2012 11:15:19.245'!
testShouldAnswerTheCollectionIfTheClassIsACollection

	(testProject typesBucket memberTypeMapFor: testClass) 
		atAllPut: (THContainerType new
					typeOfContainer: OrderedCollection new;
					typeOfElements: testClass;
					yourself).
	[self assert: [instance getClass typeOfElements = testClass].
	self assert: [instance getClass typeOfContainer isKindOf: OrderedCollection]]
		ensure: [(testProject typesBucket memberTypeMapFor: testClass) atAllPut: testClass]! !

!SInstVarMorphTest methodsFor: 'get class tests' stamp: 'tfel 2/14/2012 11:12:36.237'!
testShouldAskForAClassIfItIsUnknown
	<ignoreForTypeHarvesting>
	self
		mock: #prompt:labels:values:
		of: SMultipleChoiceRequest class
		toReturn: (Smalltalk at: #SPVTestClass) during: [
		self assert: [
			(testProject typesBucket memberTypeMapFor: testClass) atAllPut: nil.
			[self assert: [instance getClass = testClass]]
				ensure: [(testProject typesBucket memberTypeMapFor: testClass) atAllPut: testClass]]
			sends: #prompt:labels:values: to: SMultipleChoiceRequest class]! !

!SInstVarMorphTest methodsFor: 'as yet unclassified' stamp: 'ag 9/9/2011 11:49:13.354'!
testShouldBeConversibleToAnAssociationArrow
	| associationArrow |
	self mock: #openInHand of: Morph toReturn: [:i | i] during: [
	self 
		mock: #setCorrectOwnerForConnections: 
		of: SAssociationConnector 
		toReturn: [:receiver | associationArrow := receiver]
		during: [
			instance openTypeMorph.
			self assert: [associationArrow isKindOf: SAssociationConnector]]].! !

!SInstVarMorphTest methodsFor: 'as yet unclassified' stamp: 'ag 9/9/2011 11:49:13.364'!
testShouldBeConversibleToAnAssociationArrowThatHasTheCorrectOwner
	| associationArrow |
	self mock: #openInHand of: Morph toReturn: [:i | i] during: [
	self 
		mock: #setCorrectOwnerForConnections: 
		of: SAssociationConnector 
		toReturn: [:receiver | associationArrow := receiver]
		during: [
			instance openTypeMorph.
			self assert: [associationArrow owner = classMorph owner]]].! !

!SInstVarMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 2/14/2012 11:08:49.737'!
testShouldNotBecomePinnedIfDragged

	self mock: #openInHand of: Morph toReturn: [:i | i] during: [
	self mock: #setCorrectOwnerForConnections: of: SAssociationConnector toReturn: [SAssociationConnector basicNew] during: [
		instance startDrag: MorphicUnknownEvent new.
		self deny: instance isPinned]].! !

!SInstVarMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 7/12/2011 14:34:44.083'!
testShouldStayVisibleIfDragged

	self mock: #openInHand of: Morph toReturn: [:i | i] during: [
	self mock: #setCorrectOwnerForConnections: of: SAssociationConnector toReturn: [SAssociationConnector basicNew] during: [
		instance startDrag: MorphicUnknownEvent new.
		self assert: [instance wantsToBeShown]]].! !

SEntryMorphTest subclass: #SStateMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SStateMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/17/2011 15:47:58.188'!
testShouldAutomaticallyChangeItsNumberOnClassChange
	| set |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [self thereIsAStringMorphIn: set with: '2'].
	testClass addInstVarName: 'testIvar'.
	[self assert: [self 
			thereIsAStringMorphIn: set 
			with: '3']]
	ensure: [testClass removeInstVarName: 'testIvar'].! !

!SStateMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/17/2011 15:48:19.569'!
testShouldChangeItsNumberOnClassChange
	| set |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [self thereIsAStringMorphIn: set with: '2'].
	testClass addInstVarName: 'testIvar'.
	instance update.
	[self assert: [self 
			thereIsAStringMorphIn: set 
			with: '3']]
	ensure: [testClass removeInstVarName: 'testIvar'].! !

!SStateMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 2/14/2012 11:16:06.637'!
testShouldDisplayPinnedInstVarsAfterCollapse
	
	instance update.
	instance expandMorphs.
	(instance entries at: 1) togglePin.
	self assert: instance entries first isPinned.
	
	instance toggleExpansion.
	self deny: [instance expanded].
	self assert: [(instance entries at: 1) owner notNil].
	self assert: [(instance entries at: 2) owner isNil].
! !

SEntryMorphTest subclass: #STestBehaviorMorphs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

STestBehaviorMorphs subclass: #SBehaviorMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SBehaviorMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/13/2011 15:35:22.869'!
setUp

	super setUp.
	instance := klass on: testClass.! !

!SBehaviorMorphTest methodsFor: 'testing' stamp: 'tfel 4/30/2011 18:28:12.399'!
testShouldAutomaticallyChangeItsNumberOnClassChange
	| set |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [self thereIsAStringMorphIn: set with: self class testClassMethods keys size].
	testClass removeSelector: self class testClassMethods keys first.
	[self assert: [self 
			thereIsAStringMorphIn: set 
			with: self class testClassMethods keys size - 1]]
	ensure: [testClass compile: self class testClassMethods values first].! !

!SBehaviorMorphTest methodsFor: 'testing' stamp: 'tfel 4/30/2011 18:25:43.031'!
testShouldChangeItsNumberOnRefresh
	| set |
	set := self allSubmorphsRecursiveFor: instance.
	instance update.
	self assert: [self thereIsAStringMorphIn: set with: self class testClassMethods keys size].
	testClass removeSelector: self class testClassMethods keys first.
	[instance update.
	self assert: [self 
			thereIsAStringMorphIn: set 
			with: self class testClassMethods keys size - 1]]
	ensure: [testClass compile: self class testClassMethods values first].! !

!SBehaviorMorphTest methodsFor: 'testing' stamp: 'tfel 4/30/2011 19:09:54.87'!
testShouldShowCategoriesOnExpand
	| set |
	instance update.
	instance expandMorphs.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [self thereIsAStringMorphIn: set with: self class testClassMethodsCategory ].
	instance collapseMorphs.! !

!SBehaviorMorphTest methodsFor: 'testing' stamp: 'tfel 4/30/2011 18:21:10.675'!
testShouldShowTheNumberOfMethods

	instance update.
	self assert: [ self 
		thereIsAStringMorphIn: (self allSubmorphsRecursiveFor: instance) 
		with: self class testClassMethods keys size asString ].! !

STestBehaviorMorphs subclass: #SMethodCategoryMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SMethodCategoryMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 5/24/2011 14:09:36.57'!
aboutToSearchInstanceIn: aClassMorph

	super aboutToSearchInstanceIn: aClassMorph.
	self expandMorphsIn: aClassMorph.! !

!SMethodCategoryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 7/14/2011 14:46:14.386'!
expectedFailures

	^ #(testShouldUpdateTheCategoryAutomatically testShouldRemoveItselfIfCategoryIsRemoved)! !

!SMethodCategoryMorphTest methodsFor: 'running' stamp: 'tfel 5/5/2011 14:24:53.19'!
setUp

	super setUp.
	instance category: self class testClassMethodsCategory.! !

!SMethodCategoryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 19:18:34.279'!
testShouldAutomaticallyChangeItsNumberOnClassChange
	| set |
	instance update.
	set := self allSubmorphsRecursiveFor: instance.
	self assert: [self thereIsAStringMorphIn: set with: self class testClassMethods keys size].
	testClass removeSelector: self class testClassMethods keys first.
	[self assert: [self 
			thereIsAStringMorphIn: set 
			with: self class testClassMethods keys size - 1]]
	ensure: [testClass compile: self class testClassMethods values first].! !

!SMethodCategoryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 19:18:43.319'!
testShouldChangeItsNumberOnRefresh
	| set |
	set := self allSubmorphsRecursiveFor: instance.
	instance update.
	self assert: [self thereIsAStringMorphIn: set with: self class testClassMethods keys size].
	testClass removeSelector: self class testClassMethods keys first.
	[instance update.
	self assert: [self 
			thereIsAStringMorphIn: set 
			with: self class testClassMethods keys size - 1]]
	ensure: [testClass compile: self class testClassMethods values first].! !

!SMethodCategoryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/26/2011 14:35:50.999'!
testShouldRemoveItselfIfCategoryIsRemoved
	self assert: false.
	"instance update.
	self class testClassMethods keys do: [:s | testClass removeSelector: s].
	self mock: #doSilently: of: SystemChangeNotifier toReturn: [:recv :block | block value] during: [
		self
			assert: [testClass organization removeCategory: self class testClassMethodsCategory]
			sends: #delete
			to: instance class]."! !

!SMethodCategoryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/30/2011 19:12:47.556'!
testShouldShowTheCategory

	instance update.
	self assert: [self 
		thereIsAStringMorphIn: (self allSubmorphsRecursiveFor: instance)
		with: self class testClassMethodsCategory].! !

!SMethodCategoryMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/17/2011 16:18:02.705'!
testShouldUpdateTheCategoryAutomatically
	| set newCategory |
	instance update.
	set := (self allSubmorphsRecursiveFor: instance).
	newCategory := self class testClassMethodsCategory, 'Renamed'.
	self assert: [self thereIsAStringMorphIn: set with: self class testClassMethodsCategory].
	self mock: #doSilently: of: SystemChangeNotifier toReturn: [:recv :block | block value] during: [
		testClass organization 
			renameCategory: self class testClassMethodsCategory toBe: newCategory].
	[self assert: [self thereIsAStringMorphIn: set with: newCategory]]
		ensure: [testClass organization 
			renameCategory: newCategory toBe: self class testClassMethodsCategory].! !

STestBehaviorMorphs subclass: #SMethodMorphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-PathView'!

!SMethodMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 5/17/2011 15:54:14.461'!
expectedFailures

	^ #(testShouldChangeItsContentsOnRefresh)! !

!SMethodMorphTest methodsFor: 'as yet unclassified' stamp: 'tfel 4/26/2011 16:33:33.875'!
testShouldChangeItsContentsOnRefresh

	self assert: false description: 'The class canvas should accept drops from the browser'.! !

!STestBehaviorMorphs class methodsFor: 'testing' stamp: 'tfel 4/30/2011 19:04:09.148'!
isAbstract

	^ self name = #STestBehaviorMorphs! !

!STestBehaviorMorphs class methodsFor: 'test configuration' stamp: 'tfel 4/30/2011 19:04:23.027'!
testClassMethods

	^ Dictionary newFromPairs: {
		#testMethod . 'testMethod' .
		#testMethod2 . 'testMethod2'}! !

!STestBehaviorMorphs class methodsFor: 'test configuration' stamp: 'tfel 4/30/2011 19:07:11.165'!
testClassMethodsCategory

	^ 'test methods'! !

!STestBehaviorMorphs methodsFor: 'running' stamp: 'tfel 2/14/2012 11:22:38.249'!
aboutToCreateClassMorphFor: aTestClass
	"We have to add methods now, otherwise the container won't be visibile and expanding will not find it"
	self class testClassMethods keysAndValuesDo: [:methodName :methodSource |
		aTestClass compile: methodSource classified: self class testClassMethodsCategory ].! !

!SPVMorphTest class methodsFor: 'test configuration' stamp: 'tfel 4/30/2011 14:16:31.51'!
classVariableNames

	^ ''! !

!SPVMorphTest class methodsFor: 'test configuration' stamp: 'tfel 5/13/2011 18:02:00.322'!
instanceVariableNames

	^ 'ivar1 ivar2'! !

!SPVMorphTest class methodsFor: 'testing' stamp: 'tfel 4/30/2011 19:03:54.18'!
isAbstract

	^ self name = #SPVMorphTest! !

!SPVMorphTest class methodsFor: 'test configuration' stamp: 'tfel 3/6/2012 14:04:40.617'!
sandboxCategory

	^ 'Path-Tests-PathView-Sandbox'! !

!SPVMorphTest class methodsFor: 'test configuration' stamp: 'tfel 4/30/2011 14:16:15.87'!
testClassName

	^ #SPVTestClass! !

!SPVMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 4/30/2011 14:10:29.556'!
allSubmorphsRecursiveFor: aMorph
	| set flag |
	set := IdentitySet withAll: aMorph submorphs.
	flag := true.
	[ flag ]
		whileTrue: [ 
			(set select: [ :morph | morph hasSubmorphs and: [ (morph submorphs difference: set) notEmpty ] ])
				ifEmpty: [ flag := false ]
				ifNotEmptyDo: [ :nextMorphs | nextMorphs do: [ :morph | set addAll: morph submorphs ] ]	"No more submorphs to any of the morphs we have by now" ].
	^ set! !

!SPVMorphTest methodsFor: 'accessing' stamp: 'tfel 4/30/2011 18:56:39.718'!
defaultTimeout

	^ 240! !

!SPVMorphTest methodsFor: 'running' stamp: 'tfel 2/14/2012 10:47:48.448'!
setUp

	klass := (Smalltalk at: (self class name copyReplaceAll: 'Test' with: '') asSymbol).
	self
		setUpTestClass;
		setUpAssociatedTestClass;
		setUpSProject.! !

!SPVMorphTest methodsFor: 'running' stamp: 'tfel 3/6/2012 14:04:49.897'!
setUpAssociatedTestClass
	"Set up a class for associations - it and its superclasses will be the associations"
	associatedTestClass := ClassBuilder new
		superclass: Array
		subclass: (self class testClassName, 'Associated') asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self class sandboxCategory! !

!SPVMorphTest methodsFor: 'running' stamp: 'tfel 2/14/2012 13:22:54.207'!
setUpSProject

	| bucket members |
	previousProject := SProject current.
	testProject := SProject name: #SPVMorphTestProject categories: #().
	SProject 
		addProject: testProject;
		setCurrent: testProject projectName.
	bucket := THMonoTypeBucket new.
	members := bucket memberTypeMapFor: testClass.
	"Put superclasses for associatedTestClass in all instvars"
	members inject: associatedTestClass into: [:class :next |
		members at: (members indexOf: next) put: class.
		class superclass].
	testProject typesBucket: bucket.! !

!SPVMorphTest methodsFor: 'running' stamp: 'tfel 3/6/2012 14:04:19.236'!
setUpTestClass

	testClass := ClassBuilder new
		superclass: Object
		subclass: self class testClassName asSymbol
		instanceVariableNames: self class instanceVariableNames
		classVariableNames: self class classVariableNames
		poolDictionaries: ''
		category: self class sandboxCategory.! !

!SPVMorphTest methodsFor: 'running' stamp: 'tfel 2/14/2012 10:44:32.968'!
tearDown

	instance abandon.
	self tearDownSProject.
	testClass removeFromSystem.
	associatedTestClass removeFromSystem.! !

!SPVMorphTest methodsFor: 'running' stamp: 'tfel 5/5/2011 15:16:11.567'!
tearDownSProject

	SProject setCurrent: previousProject projectName.
	SProject removeProjectAt: testProject projectName asSymbol.! !

!SPVMorphTest methodsFor: 'morphic-test-helpers' stamp: 'tfel 7/12/2011 14:38:04.68'!
thereIsAStringMorphIn: aCollection with: aString
	| matchString |
	matchString :=  '*' , aString, '*'.
	^ aCollection anySatisfy: [ :morph | 
		(morph isStringMorph
			and: [matchString match: morph contents])
		or: [morph class = NCGrabbableDisplayTextMorph
			and: [matchString match: morph text asString]]]
! !

IdentityDictionary subclass: #SSBDictionary
	instanceVariableNames: 'myValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBDictionary commentStamp: 'mp 3/14/2012 16:15:45.052' prior: 0!
A SSBDictionary is blub.

Instance Variables
	myValue:		<Object>

myValue
	- xxxxx
!

!SSBDictionary class methodsFor: 'as yet unclassified' stamp: 'mp 1/5/2012 11:55:33.597'!
in: anObject

	^ self new
		myValue: anObject;
		yourself! !

!SSBDictionary methodsFor: 'accessing' stamp: 'mp 1/5/2012 11:55:12.52'!
myValue
	"Answer the value of myValue"

	^ myValue! !

!SSBDictionary methodsFor: 'accessing' stamp: 'mp 1/5/2012 11:55:12.535'!
myValue: anObject
	"Set the value of myValue"

	myValue := anObject! !

!SSBDictionary methodsFor: 'as yet unclassified' stamp: ''!
symbolFor: squeakSymbol

	^ self
		at: squeakSymbol 
		ifAbsentPut: [self myValue].! !

Object subclass: #SClassA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

!SClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/5/2010 12:54'!
methodOne

	^ true! !

!SClassA methodsFor: 'as yet unclassified' stamp: 'mp 1/22/2010 17:09'!
methodTwo: aNumber

	^ aNumber! !

Object subclass: #SPartialTraceObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Tracer'!

!SPartialTraceObject methodsFor: 'private' stamp: 'mp 3/8/2010 16:32'!
doSomething

	^ true! !

Object subclass: #SSBClassA
	instanceVariableNames: 'myValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBClassA methodsFor: 'computation' stamp: 'mp 3/8/2010 14:13'!
+ aValue

	^ self myValue + aValue! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 5/8/2012 12:34:04.516'!
complexMethod: anObject
	"blub"
	| b |
	self myValue: anObject.
	self unaryMethod 
		ifTrue: [
			b := SSBClassB on: self.
			^ b compute]
		ifFalse: [self error: 'Should not happen.'].! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:52'!
delayedMethod

	self unaryMethod
		ifTrue: [(Delay forMilliseconds: 100) wait].
	^ true! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:36'!
doNothing! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 11/5/2010 13:57:23.076'!
failingMethod: aBoolean
	self unaryMethod.
	^ aBoolean
		ifTrue: [aBoolean]
		ifFalse: [aBoolean not]! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:34'!
initialize

	self myValue: 0.! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 11/8/2010 16:56:15.457'!
manyParameters: a b: c d: e f: g long: tooLongForDisplayingAtOnce! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:48'!
methodChain: anInteger

	^ anInteger! !

!SSBClassA methodsFor: 'accessing' stamp: 'mp 3/8/2010 14:12'!
myValue
	"Answer the value of myValue"

	^ myValue! !

!SSBClassA methodsFor: 'accessing' stamp: 'mp 3/8/2010 14:26'!
myValue: anObject
	"Set the value of myValue"

	myValue := anObject.
	self changed.! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 11/5/2010 18:35:21.138'!
param: anObject method: anotherObject
	anObject == 5 ifTrue: [^ 4].
	self myValue: (Array with: anObject with: anotherObject).
	^ self myValue! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:45'!
printString

	^ self myValue asString! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:33'!
recursionMethod: aNumber

	aNumber = 0
		ifTrue: [^ 0]
		ifFalse: [
			^ aNumber + (self recursionMethod: aNumber - 1)].! !

!SSBClassA methodsFor: 'as yet unclassified' stamp: 'mp 3/9/2010 18:11'!
unaryMethod

	^ true! !

SSBClassA subclass: #SSBClassC
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBClassC methodsFor: 'as yet unclassified' stamp: 'mp 11/5/2010 14:08:00.226'!
failingMethod: aBoolean
	^ super failingMethod: aBoolean not.! !

Object subclass: #SSBClassB
	instanceVariableNames: 'myValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBClassB class methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:23'!
on: anObject

	^ self new
		myValue: anObject! !

!SSBClassB methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:25'!
compute

	self myValue unaryMethod not
		ifFalse: [^ self myValue + self myValue myValue].
	^ 0! !

!SSBClassB methodsFor: 'accessing' stamp: 'mp 3/8/2010 14:13'!
myValue
	"Answer the value of myValue"

	^ myValue! !

!SSBClassB methodsFor: 'accessing' stamp: 'mp 3/8/2010 14:13'!
myValue: anObject
	"Set the value of myValue"

	myValue := anObject! !

!SSBClassB methodsFor: 'as yet unclassified' stamp: 'mp 3/8/2010 14:27'!
update: anObject

	self myValue: anObject myValue.! !

Object subclass: #SSBClassD
	instanceVariableNames: 'myValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBClassD methodsFor: 'as yet unclassified' stamp: 'mp 2/6/2012 16:01:35.996'!
myValue2: anObject

	myValue := anObject.
	^ anObject! !

!SSBClassD methodsFor: 'as yet unclassified' stamp: 'mp 2/6/2012 16:03:16.594'!
myValue3: anObject

	myValue := anObject.
	^ anObject! !

!SSBClassD methodsFor: 'as yet unclassified' stamp: 'mp 2/6/2012 16:06:15.987'!
myValue4: anObject

	myValue := anObject.
	^ anObject! !

!SSBClassD methodsFor: 'as yet unclassified' stamp: 'mp 2/6/2012 15:51:13.002'!
myValue: anObject

	myValue := anObject.
	^ anObject! !

Object subclass: #SSBFaultLocalizationExample
	instanceVariableNames: 'mid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Sandbox'!

!SSBFaultLocalizationExample methodsFor: 'accessing' stamp: 'mp 9/21/2011 14:34:38.815'!
mid
	"Answer the value of mid"

	^ mid! !

!SSBFaultLocalizationExample methodsFor: 'accessing' stamp: 'mp 9/21/2011 14:34:38.83'!
mid: anObject
	"Set the value of mid"

	mid := anObject! !

!SSBFaultLocalizationExample methodsFor: 'computation' stamp: 'mp 9/21/2011 14:35:32.022'!
midX: firstNumber Y: secondNumber Z: thirdNumber
	self mid: thirdNumber.
	secondNumber < thirdNumber
		ifTrue: [firstNumber < secondNumber
					ifTrue: [self mid: secondNumber]
					ifFalse: [firstNumber < thirdNumber
								ifTrue: [self mid: secondNumber]]]
		ifFalse: [firstNumber > secondNumber
					ifTrue: [self mid: secondNumber]
					ifFalse: [firstNumber > thirdNumber
								ifTrue: [self mid: firstNumber]]]! !

Object subclass: #THSBClassA
	instanceVariableNames: 'a b c'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester-Sandbox'!

!THSBClassA class methodsFor: 'as yet unclassified' stamp: 'mp 5/12/2011 14:38:46.168'!
a: anObjectA b: anObjectB c: anObjectC

	^ self new
		a: anObjectA;
		b: anObjectB;
		c: anObjectC;
		yourself.! !

!THSBClassA methodsFor: 'accessing' stamp: 'mp 5/26/2010 16:25'!
a
	^ a! !

!THSBClassA methodsFor: 'accessing' stamp: 'mha 4/27/2010 23:12'!
a: x
	a := x! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 17:16'!
addValueToA: aNumber

	^ a + aNumber! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/3/2011 10:09:49.679'!
assoc: anAssoc collection: aCollection dict: aDictionary number: aNumber
	| assoc coll dict num |
	assoc := anAssoc.
	coll := aCollection.
	dict := aDictionary.
	num := aNumber.
	self b: assoc.
	self b: coll.
	self b: dict.
	self b: num.
	^ {assoc. coll. dict. num}! !

!THSBClassA methodsFor: 'accessing' stamp: 'mha 4/27/2010 23:12'!
b
	^ b! !

!THSBClassA methodsFor: 'accessing' stamp: 'mha 4/27/2010 23:12'!
b: x
	b := x! !

!THSBClassA methodsFor: 'accessing' stamp: 'mha 4/27/2010 23:12'!
c
	^ c! !

!THSBClassA methodsFor: 'accessing' stamp: 'mha 4/27/2010 23:12'!
c: x
	c := x! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 16:18'!
collectCollection: aCollection
	"Enforce a closure, and force the closure to access external state."
	
	^ aCollection collect: [ :each | | oink gunk | aCollection includes: each ]! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 17:54'!
concatTwoStrings: aBlock

	^ aBlock value: 'Hello' value: 'world'! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/27/2010 16:54'!
evalBlock: aBlock with: anObject

	aBlock value: anObject.
	^ #ShouldNotBeReached! !

!THSBClassA methodsFor: 'initialize-release' stamp: 'mp 5/26/2010 16:48'!
initialize

	self a: 1.! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 16:18'!
locals: val which: i
	| local mocal nocal r |
	i = 1 ifTrue: [ local := val. r := local ].
	i = 2 ifTrue: [ mocal := val. r := mocal ].
	i = 3 ifTrue: [ nocal := val. r := nocal ].
	^ r! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 16:18'!
numbers: n
	^ n! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/27/2010 16:53'!
returnBlock: anObject

	self evalBlock: [:each | ^ each] with: anObject! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/27/2010 17:26'!
returnTrue

	self. "Do nothing --- prevent quick return accessor"
	^ true! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 17:36'!
sayHelloTo: aString
	| obj |
	obj := THSBClassB new.
	^ obj addExclamationMarkTo: (obj getHello, Character space, aString)! !

!THSBClassA methodsFor: 'methods' stamp: 'mp 5/26/2010 16:18'!
strings: s magnitudes: m andObjects: o index: i
	^ { s. m. o } at: i! !

Object subclass: #THSBClassB
	instanceVariableNames: 'mark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester-Sandbox'!

!THSBClassB methodsFor: 'methods' stamp: 'mp 5/26/2010 17:40'!
addExclamationMarkTo: aString
	^ aString, mark! !

!THSBClassB methodsFor: 'methods' stamp: 'mp 5/26/2010 17:37'!
getHello

	^ 'Hello'! !

!THSBClassB methodsFor: 'initialize-release' stamp: 'mp 5/26/2010 17:40'!
initialize

	self mark: '!!'.! !

!THSBClassB methodsFor: 'accessing' stamp: 'mp 5/26/2010 17:39'!
mark
	"Answer the value of mark"

	^ mark! !

!THSBClassB methodsFor: 'accessing' stamp: 'mp 5/26/2010 17:39'!
mark: anObject
	"Set the value of mark"

	mark := anObject! !

THSBClassB subclass: #THSBClassSubB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester-Sandbox'!

THSBClassB subclass: #THSBClassSubB2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-Harvester-Sandbox'!

!THSBClassSubB2 methodsFor: 'as yet unclassified' stamp: 'mp 5/12/2011 14:57:58.156'!
initialize

	super initialize.
	self mark: '?'.
	1 = 3
		ifTrue: [self mark: 3 + 4.
				self mark: 5 + 6]! !

MwMethodWrapper subclass: #SAroundWrapperMethodOne
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

!SAroundWrapperMethodOne methodsFor: 'evaluating' stamp: 'mp 3/5/2010 12:48'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	self beforeMethod.
	^ [
		false
	] ensure: [self afterMethod]! !

MwMethodWrapper subclass: #SAroundWrapperMethodTwo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

!SAroundWrapperMethodTwo methodsFor: '-- all --' stamp: 'mp 3/5/2010 12:51'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	"This is the general case where you want both a before and after method, but if you
	want just a before method, you might want to override this method for optimization."

	self beforeMethod.
	^ [
		(self clientMethod valueWithReceiver: anObject arguments: #(2)) + 1.
	] ensure: [self afterMethod]! !

MwMethodWrapper subclass: #SBeforeWrapper
	instanceVariableNames: 'touched'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Tests-MethodWrappers'!

!SBeforeWrapper methodsFor: 'evaluating' stamp: 'mp 2/22/2010 09:32'!
beforeMethod

	self touched: self touched + 1.! !

!SBeforeWrapper methodsFor: '-- all --' stamp: 'mp 1/22/2010 14:58'!
initialize

	self touched: 0.! !

!SBeforeWrapper methodsFor: 'accessing' stamp: 'mp 1/22/2010 14:41'!
touched
	"Answer the value of touched"

	^ touched! !

!SBeforeWrapper methodsFor: 'accessing' stamp: 'mp 1/22/2010 14:41'!
touched: anObject
	"Set the value of touched"

	touched := anObject! !
